#include <stdio.h>
#include <goa/menu.h>

#include "lucien.h"
#include "config.h"



// config_entry

config_entry::config_entry(char *name,char *description)
{
	n=name;
	desc=description;
}

void config_entry::read(config *c) {}

void config_entry::write(config *c) {}

void config_entry::make_menu(m_menu *mn) {}

void config_entry::read_menu() {}



// int_entry

int_entry::int_entry(char *name,int *val,int def,char *description)
:config_entry(name,description)
{
	v=val;
	d=def;
}

void int_entry::read(config *c)
{
	*v=c->get_int(n,d);
}

void int_entry::write(config *c)
{
	c->set_int(n,*v);
}

void int_entry::make_menu(m_menu *mn) {}

void int_entry::read_menu() {}



// double_entry

double_entry::double_entry(char *name,double *val,double def,char *description)
:config_entry(name,description)
{
	v=val;
	d=def;
}

void double_entry::read(config *c)
{
	//*v=c->get_int(n,d);
}

void double_entry::write(config *c)
{
	//c->set_int(n,*v);
}

void double_entry::make_menu(m_menu *mn) {}

void double_entry::read_menu() {}



// string_entry

string_entry::string_entry(char *name,char **val,char *def,char *description)
:config_entry(name,description)
{
	v=val;
	d=def;
}

void string_entry::read(config *c)
{
	//*v=c->get_int(n,d);
}

void string_entry::write(config *c)
{
	//c->set_int(n,*v);
}

void string_entry::make_menu(m_menu *mn) {}

void string_entry::read_menu() {}



// bool_entry

bool_entry::bool_entry(char *name,bool *val,bool def,
char *description,char *positive,char *negative)
:config_entry(name,description)
{
	v=val;
	d=def;
	pos=positive;
	neg=negative;
}

void bool_entry::read(config *c)
{
	*v=c->get_int(n,d?1:0)?true:false;
}

void bool_entry::write(config *c)
{
	c->set_int(n,*v?1:0);
}

void bool_entry::make_menu(m_menu *mn)
{
	if(desc)
	{
		m=new m_multi(desc,0,neg,pos,0);
		m->i=*v?1:0;

		mn->insert(m,0);
	}
}

void bool_entry::read_menu()
{
	*v=m->i?true:false;
}



// key_entry

key_entry::key_entry(char *name,int *val,int def,char *description)
:int_entry(name,val,def,description)
{
}

void key_entry::make_menu(m_menu *mn)
{
	if(desc)
	{
		m=new m_key(desc,*v,0);

		mn->insert(m,0);
	}
}

void key_entry::read_menu()
{
	*v=m->k;
}




// multi_entry

multi_entry::multi_entry(char *name,int *val,int def,
int numvalues,config_multi *values,char *description)
:int_entry(name,val,def,description)
{
	num=numvalues;
	vals=values;
}

void multi_entry::make_menu(m_menu *mn)
{
	m=new m_multi(desc,0,num);

	for(int i=0;i<num;i++)
	m->insert(vals[i].name);

	int i;

	for(i=0;i<num;i++) if(vals[i].value==*v) break;
	if(i==num) for(i=0;i<num;i++) if(vals[i].value==d) break;
	if(i==num) i=0;

	m->i=i;

	mn->insert(m,0);
}

void multi_entry::read_menu()
{
	*v=vals[m->i].value;
}



// resolution_entry

resolution_entry::resolution_entry(char *name,int *width,int *height,int defwidth,int defheight,
int numresolutions,config_resolution *resolutions,char *description)
:config_entry(name,description)
{
	w=width; h=height;
	defw=defwidth; defh=defheight;

	num=numresolutions;
	res=resolutions;
}

void resolution_entry::read(config *c)
{
	char str[256];

	sprintf(str,"%swidth",n?n:"");
	*w=c->get_int(str,defw);
	sprintf(str,"%sheight",n?n:"");
	*h=c->get_int(str,defh);
}

void resolution_entry::write(config *c)
{
	char str[256];

	sprintf(str,"%swidth",n?n:"");
	c->set_int(str,*w);
	sprintf(str,"%sheight",n?n:"");
	c->set_int(str,*h);
}

void resolution_entry::make_menu(m_menu *mn)
{
	m=new m_multi(desc,0,num);

	for(int i=0;i<num;i++)
	m->insert(res[i].name);

	int i;

	for(i=0;i<num;i++) if(res[i].width==*w&&res[i].height==*h) break;
	if(i==num) for(i=0;i<num;i++) if(res[i].width==defw&&res[i].height==defh) break;
	if(i==num) i=0;

	m->i=i;

	mn->insert(m,0);
}

void resolution_entry::read_menu()
{
	*w=res[m->i].width;
	*h=res[m->i].height;
}




// spacer_entry

spacer_entry::spacer_entry(int space):config_entry(0)
{
	s=space;
}

void spacer_entry::make_menu(m_menu *mn)
{
	mn->insert(new g_spacer(0,s),0);
}

void spacer_entry::read_menu()
{
}



// config

config::config(char *section)
{
	file=0;
	sec=section;
}

config::~config()
{
	save();

	entries.clear();
}

void config::add(config_entry *entry)
{
	entries.add_tail(entry);
}


int config::get_int(char *name,int def)
{
	return(get_config_int(sec,name,def));
}

const char *config::get_string(char *name,char *def)
{
	return(get_config_string(sec,name,def));
}

void config::set_int(char *name,int val)
{
	set_config_int(sec,name,val);
}

void config::set_string(char *name,char *val)
{
	set_config_string(sec,name,val);
}



void config::load(char *filename)
{
	file=filename;
	set_config_file(file);

	iterator i(&entries);
	config_entry *e;
	while(e=(config_entry *)i.step())
	e->read(this);
}

void config::save()
{
	save(file);
}

void config::save(char *filename)
{
	file=filename;
	if(!file) return; // hmm...
	set_config_file(file);

	iterator i(&entries);
	config_entry *e;
	while(e=(config_entry *)i.step())
	e->write(this);
}

void config::make_menu(m_menu *mn)
{
	iterator i(&entries);
	config_entry *e;
	while(e=(config_entry *)i.step())
	{
		e->make_menu(mn);
	}
}

void config::read_menu()
{
	iterator i(&entries);
	config_entry *e;
	while(e=(config_entry *)i.step())
	{
		e->read_menu();
	}

	save();
}
