#ifndef _LUCIEN_CONFIG_H_
#define _LUCIEN_CONFIG_H_

#include <goa/menu.h>
#include <goa/list.h>

class config;
class configmenu;

class config_entry:public node
{
	public:
	config_entry(char *name,char *description=0);

	virtual void read(config *c);
	virtual void write(config *c);

	virtual void make_menu(m_menu *mn);
	virtual void read_menu();

	char *n,*desc;
};

class int_entry:public config_entry
{
	public:
	int_entry(char *name,int *val,int def=0,char *description=0);

	void read(config *c);
	void write(config *c);

	void make_menu(m_menu *mn);
	void read_menu();

	int *v,d;
};

class double_entry:public config_entry
{
	public:
	double_entry(char *name,double *val,double def=0,char *description=0);

	void read(config *c);
	void write(config *c);

	void make_menu(m_menu *mn);
	void read_menu();

	double *v,d;
};

class string_entry:public config_entry
{
	public:
	string_entry(char *name,char **val,char *def=0,char *description=0);

	void read(config *c);
	void write(config *c);

	void make_menu(m_menu *mn);
	void read_menu();

	char **v,*d;
};

class bool_entry:public config_entry
{
	public:
	bool_entry(char *name,bool *val,bool def=false,
	char *description=0,char *positive="True",char *negative="False");

	void read(config *c);
	void write(config *c);

	void make_menu(m_menu *mn);
	void read_menu();

	bool *v,d;
	char *pos,*neg;
	m_multi *m;
};

class key_entry:public int_entry
{
	public:
	key_entry(char *name,int *val,int def=0,char *description=0);

	void make_menu(m_menu *mn);
	void read_menu();

	m_key *m;
};

struct config_multi
{
	int value;
	char *name;
};

class multi_entry:public int_entry
{
	public:
	multi_entry(char *name,int *val,int def,
	int numvalues,config_multi *values,char *description);

	void make_menu(m_menu *mn);
	void read_menu();

	int num;
	config_multi *vals;

	m_multi *m;
};

struct config_resolution
{
	int width,height;
	char *name;
};

class resolution_entry:public config_entry
{
	public:
	resolution_entry(char *name,int *width,int *height,int defwidth,int defheight,
	int numresolutions,config_resolution *resolutions,char *description);

	void read(config *c);
	void write(config *c);

	void make_menu(m_menu *mn);
	void read_menu();

	int *w,*h,defw,defh;
	int num;
	config_resolution *res;

	m_multi *m;
};



class spacer_entry:public config_entry
{
	public:
	spacer_entry(int space);

	void make_menu(m_menu *mn);
	void read_menu();

	int s;
};



class config
{
	public:
	config(char *section=0);
	~config();

	void add(config_entry *entry);

	int get_int(char *name,int def);
	const char *get_string(char *name,char *def);
	void set_int(char *name,int val);
	void set_string(char *name,char *val);

	void load();
	void load(char *filename);
	void save();
	void save(char *filename);

	void make_menu(m_menu *mn);
	void read_menu();

	char *file,*sec;

	list entries;
};

#endif
