#include <time.h>
#include <string.h>
#include <goa/allegro.h>
#include <goa/input.h>
#include <goa/menu.h>

#include "main.h"
#include "game.h"
#include "swearconfig.h"




static double smoothfactor[]={0,10,6,3,1};

static double trackingangles[]={PI/6,PI/4,PI/3,PI/2};

game *g;



game::game()
{
	playing=false;
}

game::~game()
{
}



void game::setup()
{
	t=0;
	score=0;
	firstframe=true;
}

void game::play()
{
	bool esc=true;

	synctime();

	playing=true;
	while(playing)
	{
		live();
		draw();

		blit(bm,screen,0,0,0,0,SCREEN_W,SCREEN_H);

		if(key[KEY_ESC])
		{
			if(!esc)
			{
				fadedown();
				return;
			}
		}
		else esc=false;
	}

	fadedown();

	dohighscore();
}

void game::live()
{
	if(firstframe||c->smoothing==0)
	{
		//x_axis=left;
//		y_axis=up;
//		z_axis=toward;

		cam_current=cam;

		firstframe=false;
	}
	else
	{
		// gaah.

//		cam_current=cam;

//		double fac=smoothfactor[c->smoothing];
		double fac=1;

		vector x_axis=cam.x_axis();
		vector y_axis=cam.y_axis();
		vector z_axis=cam.z_axis();
		vector pos=cam.position();

		vector x_axis_old=cam_current.x_axis();
		vector y_axis_old=cam_current.y_axis();
		vector z_axis_old=cam_current.z_axis();
		vector pos_old=cam_current.position();

		vector x_axis_new,y_axis_new,z_axis_new,pos_new;

		x_axis_new=x_axis;
		y_axis_new=y_axis;
		z_axis_new=z_axis;
		pos_new=pos;

		cam_current=matrix(x_axis_new,y_axis_new,z_axis_new,pos_new);

//		double fac=smoothfactor[c->smoothing];

/*		double speed=10,length;
		vector delta;

		delta=up-y_axis;
		length=delta.length();
		if(length>speed*dt) delta*=speed*dt/length;
		y_axis+=delta;
		y_axis.normalize();

		delta=toward-z_axis;
		length=delta.length();
		if(length>speed*dt) delta*=speed*dt/length;
		z_axis+=delta;
		z_axis-=y_axis*y_axis.dot(z_axis);
		z_axis.normalize();*/
	}

	cam_inv=cam_current.fast_inverse();
}

void game::draw()
{
	textprintf(bm,victor_12,SCREEN_W-100,SCREEN_H-13,1,"Score: %d",score);

	if(score>p->high[gamenum][0].score)
	textprintf(bm,victor_12,2,SCREEN_H-13,2,"High score: %d by YOU!",score);
	else
	textprintf(bm,victor_12,2,SCREEN_H-13,1,"High score: %d by %s",p->high[gamenum][0].score,p->high[gamenum][0].name);
}



void game::dohighscore()
{
	if(p->ishighscore(gamenum,score))
	{
		int time_now=int(time(0));

		m_text *mt;
		m_string *mstr;
		m_menu *m=new m_menu(
			mt=new m_text("High score!",0,&look_ni),
			mstr=new m_string("Enter name: ",100,43,1,0),
			0
		);
		mt->clear_flag(GUIFLAG_ACTIVE);
		m->place=GUIPLACE_CENTER|GUIPLACE_MIDDLE;
		m->set_flag(GUIFLAG_NOMOUSE);

		do { m->run(); }
		while(!strlen(mstr->str));

		p->showhighscores(gamenum,p->inserthighscore(gamenum,mstr->str,score,int(t),time_now));
		p->savehighscores();

		delete m;
	}
}



void game::synctime()
{
	sync_delta_time();
}

void game::calctime()
{
	dt=get_delta_time();
	t+=dt;
}

void game::fadedown()
{
	acquire_bitmap(screen);
	for(int y=0;y<SCREEN_H;y++)
	for(int x=0;x<SCREEN_W;x++)
	{
		int sx=x;
		int sy=3*y%SCREEN_H+3*y/SCREEN_H;

		int col=getpixel(screen,sx,sy);
		if(col>=64) col=(col&0xc0)+((col&0x3f)>>1);
		else if(col>=32) col=(col&0xe0)+((col&0x1f)>>1);
		putpixel(screen,sx,sy,col);
	}
	release_bitmap(screen);
}

int game::project(vector r,const vector &norm,int &x_s,int &y_s,bool onesided)
{
	vector rot;
	int col;

	rot=cam_inv*r;

	double sc=CAMERA_SCALE/rot.z;

	if(sc<0||sc>CAMERA_CUTOFF) return(-1);

	x_s=int(rot.x*sc+double(SCREEN_W-1)/2);
	y_s=int(rot.y*sc+double(SCREEN_H-1)/2);

	col=int(-(norm.dot(cam_current.z_axis()))*64);

	if(onesided) col=ABS(col);

	return(col>0?col:0);
}



// wormgame

wormgame::wormgame():game()
{
}

void wormgame::live()
{
	switch(c->cameramode)
	{
		case CMODE_REVOLVING:
			cam=matrix::translation(0,0,-CAMERA_REVOLVE_DIST);
			cam=matrix(w.dir*w.normal,-w.dir,-w.normal,vector(0,0,0))*cam; // right, down, forward
		break;

		case CMODE_TRACKING:
		{
			double a=trackingangles[c->trackingangle];
			double sin_a=sin(a),cos_a=cos(a);

			vector forward=w.dir*cos_a-w.normal*sin_a;
			vector down=-w.dir*sin_a-w.normal*cos_a;

			cam=matrix::translation(0,0,-CAMERA_FOLLOW_DIST);
			cam=matrix(down*forward,down,forward,vector(0,0,0))*cam; // right, down, forward
			cam=matrix::translation(w.pos)*cam;
		}
		break;

		case CMODE_SLIDING:
			cam=matrix(w.dir*w.normal,-w.dir,-w.normal,vector(0,0,0)); // right, down, forward
			cam=cam*matrix::translation(cam.x_axis().dot(w.pos),cam.y_axis().dot(w.pos),-CAMERA_SLIDE_DIST);
//			cam=matrix::translation(w.pos)*cam;
//			cam=matrix::translation(cam.z_axis()*(cam.position().dot(cam.z_axis())-CAMERA_SLIDE_DIST))*cam;
		break;
	}

	game::live();
}




// simplegame

simplegame::simplegame():wormgame()
{
	gamenum=0;
}

void simplegame::setup()
{
	wormgame::setup();

	w.place();
	a.place();
}

void simplegame::live()
{
	calctime();

	w.live();
	a.live();

	if(a.collide(w.pos,w.normal))
	{
		score+=a.points;
		w.len+=SIMPLE_APPLEFACTOR*double(a.points);
		a.place();
	}

	wormgame::live();
}

void simplegame::draw()
{
	clear(bm);

	drawing_mode(DRAW_MODE_TRANS,0,0,0);

	p->draw();

	w.draw();
	a.draw();

	solid_mode();

	textprintf(bm,victor_12,SCREEN_W-100,SCREEN_H-25,1,"Next: %d",a.points);

	wormgame::draw();
}



// huntgame

huntgame::huntgame():wormgame()
{
	gamenum=1;
	a.w=&w;
}

void huntgame::setup()
{
	wormgame::setup();

	w.place();
	a.place();
}

void huntgame::live()
{
	calctime();

	w.live();
	a.live();

	if(a.collide(w.pos,w.normal))
	{
		score+=a.points;
		w.len+=HUNT_APPLEFACTOR*double(a.points);
		a.place();
	}

	wormgame::live();
}

void huntgame::draw()
{
	clear(bm);

	drawing_mode(DRAW_MODE_TRANS,0,0,0);

	p->draw();

	w.draw();
	a.draw();

	solid_mode();

	textprintf(bm,victor_12,SCREEN_W-100,SCREEN_H-25,1,"Next: %d",a.points);

	wormgame::draw();
}



// blindgame

blindgame::blindgame()
{
	gamenum=2;
}

void blindgame::draw()
{
	clear(bm);

	drawing_mode(DRAW_MODE_TRANS,0,0,0);

	s.live();
	s.draw();

//	p->draw();

	w.draw();
	a.draw();

	solid_mode();

	textprintf(bm,victor_12,SCREEN_W-100,SCREEN_H-25,1,"Next: %d",a.points);

	game::draw();
}
