#include <stdio.h>
#include <allegro.h>
#include <allegro/internal/aintern.h>

#include <goa/font.h>
#include <goa/types.h>



static void _remap(BITMAP *bm,unsigned char *table)
{
	for(int y=0;y<bm->h;y++)
	{
		unsigned char *data=bm->line[y];
		int n=bm->w;
		while(n--) { *data=table[*data]; data++; }
	}
}

void prepare_colour_font(FONT *f,PALETTE pal)
{
	if(bitmap_color_depth(screen)==8)
	{
		unsigned char table[256];

		table[0]=0;
		for(int i=1;i<256;i++)
		table[i]=makecol8(pal[i].r<<2,pal[i].g<<2,pal[i].b<<2);

		FONT_COLOR_DATA *fd=(FONT_COLOR_DATA *)f->data;

		// no unicode support!
		for(int i=0;i<fd->end-fd->begin;i++) // end is one-past
		{
			_remap(fd->bitmaps[i],table);
		}
	}
}

void set_colour_font_palette(PALETTE pal)
{
	if(bitmap_color_depth(screen)!=8) set_palette(pal);
}



typedef struct amiga_glyph
{
	short kern;
	FONT_GLYPH glyph;
} amiga_glyph;

typedef struct font_amiga_data
{
	int begin,end; // first char and LAST char
	amiga_glyph **glyphs;
} font_amiga_data;

static amiga_glyph *_amiga_find_glyph(AL_CONST FONT *f,int ch)
{
    font_amiga_data *af=(font_amiga_data *)(f->data);

	if(ch>=af->begin&&ch<=af->end) return(af->glyphs[ch-af->begin]);

    if(ch!=allegro_404_char) return(_amiga_find_glyph(f,allegro_404_char));
    return(0);
}

static int amiga_font_height(AL_CONST FONT *f)
{
   return(f->height);
}

static int amiga_length(AL_CONST FONT *f,AL_CONST char *text)
{
	AL_CONST char *p=text;
	int ch=0,w=0;
	amiga_glyph *g;

	while(ch=ugetxc(&p))
	{
		g=_amiga_find_glyph(f,ch);
		w+=g->glyph.w+g->kern;
	}
	w-=g->kern;

	return(w);
}

static int amiga_char_length(AL_CONST FONT *f,int ch)
{
    amiga_glyph *g=_amiga_find_glyph(f,ch);
    return(g?g->glyph.w:0);
}

static int amiga_render_char(AL_CONST FONT *f,int ch,int fg,int bg,BITMAP *bmp,int x,int y)
{
	int w=0;
	amiga_glyph *g=0;

	int old_textmode=_textmode;
	_textmode=bg;

	g=_amiga_find_glyph(f,ch);

	acquire_bitmap(bmp);

	if(g)
	{
		bmp->vtable->draw_glyph(bmp,&g->glyph,x,y,fg,bg);
		w=g->glyph.w+g->kern;
	}

	release_bitmap(bmp);
	_textmode=old_textmode;

	return(w);
}

static void amiga_render(AL_CONST FONT *f,AL_CONST char *text,int fg,int bg,BITMAP *bmp,int x,int y)
{
	int ch=0;
	AL_CONST char *p=text;

	int old_textmode=_textmode;
	_textmode=bg;

	acquire_bitmap(bmp);

	while((ch=ugetxc(&p)))
	{
		x+=f->vtable->render_char(f,ch,fg,bg,bmp,x,y);
	}

	release_bitmap(bmp);
	_textmode=old_textmode;
}

static void amiga_destroy(FONT* f)
{
	font_amiga_data *af=0;
	int i=0;

	if(!f) return;

	af=(font_amiga_data *)(f->data);

	for(i=af->begin;i<=af->end;i++) free(af->glyphs[i-af->begin]);

	free(af->glyphs);
	free(af);

	free(f);
}

FONT_VTABLE _font_vtable_amiga=
{
	amiga_font_height,
	amiga_char_length,
	amiga_length,
	amiga_render_char,
	amiga_render,
	amiga_destroy
};

FONT_VTABLE* font_vtable_amiga=&_font_vtable_amiga;




int8 getbyte(uint8 *ptr) { return(*ptr); }
uint8 getubyte(uint8 *ptr) { return(*ptr); }
int16 getword(uint8 *ptr) { return((ptr[0]<<8)|ptr[1]); }
uint16 getuword(uint8 *ptr) { return((ptr[0]<<8)|ptr[1]); }
int32 getlong(uint8 *ptr) { return((ptr[0]<<24)|(ptr[1]<<16)|(ptr[2]<<8)|ptr[3]); }
uint32 getulong(uint8 *ptr) { return((ptr[0]<<24)|(ptr[1]<<16)|(ptr[2]<<8)|ptr[3]); }

/*struct TextFont
{
	struct Message tf_Message;  // reply message for font removal
						  // font name in LN        |  used in this
	UWORD   tf_YSize;      // font height            |  order to best
	UBYTE   tf_Style;      // font style             |  match a font
	UBYTE   tf_Flags;      // preferences and flags  |  request.
	UWORD   tf_XSize;      // nominal font width
	UWORD   tf_Baseline;   // distance from the top of char to baseline
	UWORD   tf_BoldSmear;  // smear to affect a bold enhancement

	UWORD   tf_Accessors;  // access count

	UBYTE   tf_LoChar;     // the first character described here
	UBYTE   tf_HiChar;     // the last character described here
	UBYTE  *tf_CharData;   // the bit character data

	UWORD   tf_Modulo;     // the row modulo for the strike font data
	ULONG  *tf_CharLoc;    // ptr to location data for the strike font
						  //   2 words: bit offset then size
	WORD   *tf_CharSpace;  // ptr to words of proportional spacing data
	WORD   *tf_CharKern;   // ptr to words of kerning data
};*/

void put_bitmap_pixel(uint8 *bitmap,int x) { bitmap[x>>3]|=1<<((~x)&7); }
bool get_bitmap_pixel(uint8 *bitmap,int x) { return((bitmap[x>>3]&1<<((~x)&7))?true:false); }

FONT *load_amiga_font(char *filename)
{
	FILE *fh=fopen(filename,"rb");
	if(!fh) return(0);

	fseek(fh,0,SEEK_END);
	int size=ftell(fh);
	fseek(fh,0,SEEK_SET);

	uint8 *ff=(uint8 *)malloc(size);
	if(!ff) return(0);

	int res=fread(ff,size,1,fh);

	fclose(fh);

	if(res!=1) { free(ff); return(0); }

	int ysize=getuword(ff+110);
	int style=getubyte(ff+112);
	int flags=getubyte(ff+113);
	int xsize=getuword(ff+114);
	//int baseline=getuword(116);
	//int boldsmear=getuword(118);
	//int accessors=getuword(120);

	int lochar=getubyte(ff+122);
	int hichar=getubyte(ff+123);

	int chardata=getulong(ff+124);

	int modulo=getuword(ff+128);
	int charloc=getulong(ff+130);
	int charspace=getulong(ff+134);
	int charkern=getulong(ff+138);

	// allocate space for the font data
	int numchars=hichar-lochar+1;

	FONT *f=(FONT *)calloc(1,sizeof(FONT));
	font_amiga_data *af=(font_amiga_data *)calloc(1,sizeof(font_amiga_data));
	amiga_glyph **g=(amiga_glyph **)calloc(numchars,sizeof(amiga_glyph *));

	af->begin=lochar;
	af->end=hichar;
	af->glyphs=g;

	f->data=(void *)af;
	f->height=ysize;
	f->vtable=font_vtable_amiga;

	for(int i=0;i<numchars;i++)
	{
		int width=getuword(ff+34+charloc+i*4);
		int bytewidth=(width+7)/8;

		g[i]=(amiga_glyph *)calloc(1,sizeof(amiga_glyph)+bytewidth*ysize);
		g[i]->glyph.w=width;
		g[i]->glyph.h=ysize;

		if(flags&0x20) // FPF_PROPORTIONAL
		g[i]->kern=getword(ff+32+charkern+i*2)+getuword(ff+32+charspace+i*2)-width;
		else
		g[i]->kern=xsize-width;

		int charpos=getuword(ff+32+charloc+i*4);

/*if(i=='t'-lochar)
{
char str[1024];
sprintf(str,"i%d w%d bw%d k%d sp%d kr%d",i,width,bytewidth,g[i]->kern,getuword(ff+32+charspace+i*2),getword(ff+32+charkern+i*2));
alert("bah",str,0,"duh",0,0,0);
}*/

		for(int y=0;y<ysize;y++)
		for(int x=0;x<width;x++)
		{
			if(get_bitmap_pixel(ff+32+chardata+y*modulo,x+charpos))
			put_bitmap_pixel(g[i]->glyph.dat+y*bytewidth,x);
		}
	}
	return(f);
}
