#include <allegro.h>
#include <string.h>

#include <goa/gui.h>
#include <goa/input.h>
#include <goa/clip.h>



static g_frame defaultframe;



g_gadget::g_gadget()
{
	place=GUIPLACE_CENTER|GUIPLACE_MIDDLE;
	flags=0;
	fixwidth=fixheight=0;
	ev=0;
	frame=&defaultframe;

	minwidth=minheight=maxwidth=maxheight=0;

	parent=0;
	pressing=false;
	ext=0;

	set_flag(GUIFLAG_ACTIVE);
}

g_gadget::~g_gadget()
{
	delete ext;
}

void g_gadget::become_child(g_gadget *newparent)
{
	parent=newparent;
}

void g_gadget::calc_sizes()
{
	minwidth=maxwidth=frame->left+frame->right;
	minheight=maxheight=frame->top+frame->bottom;
}

void g_gadget::resize(int w,int h)
{
}

void g_gadget::draw(BITMAP *bm,int x,int y,int w,int h)
{
	int x2=x+w-1,y2=y+h-1;

	if(get_flag(GUIFLAG_PRESSED)) frame->draw_pressed(bm,x,y,x2,y2);
	else if(get_flag(GUIFLAG_FOCUSED)) frame->draw_focused(bm,x,y,x2,y2);
	else if(!get_flag(GUIFLAG_ACTIVE)) frame->draw_inactive(bm,x,y,x2,y2);
	else frame->draw_normal(bm,x,y,x2,y2);
}

bool g_gadget::key(int c,int k)
{
	switch(k)
	{
		case KEY_ENTER:
			trigger(GUITRIGGER_KEY);
			return(true);

		case KEY_ESC:
			event(GUIRES_EXIT);
			return(true);
	}

	return(false);
}

bool g_gadget::mouse_click(int x,int y,int b,int ch)
{
	if(!get_flag(GUIFLAG_CLICKABLE)) return(false);

	/*if(get_flag(GUIFLAG_TOGGLE))
	{
		if(get_flag(GUIFLAG_PRESSED)) clear_flag(GUIFLAG_PRESSED);
		else set_flag(GUIFLAG_PRESSED);
	}
	else*/ set_flag(GUIFLAG_PRESSED);


	pressing=true;
	redraw();
	return(true);
}

bool g_gadget::mouse_release(int x,int y,int b,int ch)
{
	if(!get_flag(GUIFLAG_CLICKABLE)) return(false);

	if(b==0)
	{
		if(get_flag(GUIFLAG_PRESSED))
		{
			clear_flag(GUIFLAG_PRESSED);
			trigger(ch&2?GUITRIGGER_RIGHT:GUITRIGGER_LEFT);
			redraw();
		}
		pressing=false;
	}
	return(true);
}

void g_gadget::mouse_move(int x,int y)
{
}

void g_gadget::mouse_in(int x,int y)
{
	if(pressing) { set_flag(GUIFLAG_PRESSED); redraw(); }
}

void g_gadget::mouse_out(int x,int y)
{
	if(pressing) { clear_flag(GUIFLAG_PRESSED); redraw(); }
}

void g_gadget::trigger(int type)
{
	if(ev) event(ev);
}

bool g_gadget::want_focus()
{
	return(get_flag(GUIFLAG_ACTIVE));
}

bool g_gadget::release_focus()
{
	return(!pressing);
}

void g_gadget::get_focus()
{
	set_flag(GUIFLAG_FOCUSED);
	redraw();
}

void g_gadget::lose_focus()
{
	clear_flag(GUIFLAG_FOCUSED);
	redraw();
}

void g_gadget::navigate_into(int type)
{
}

void g_gadget::redraw()
{
//	for(int i=0;i<n;i++) m->clear_flag(GUIFLAG_REDRAW);
	if(ext)
	{
		set_flag(GUIFLAG_REDRAW);
	}
	else parent->redraw();
}

void g_gadget::event(int event)
{
	if(ext)
	{
		ext->event=event;
	}
	else parent->event(event);
}

bool g_gadget::get_flag(int val) { return((flags&val)?true:false); }
void g_gadget::set_flag(int val) { flags|=val; }
void g_gadget::clear_flag(int val) { flags&=~val; }

int g_gadget::run(int x0,int y0)
{
	int res;

	if(startup(x0,y0))
	{
		res=wait_events();

		shutdown();

		return(res);
	}

	return(GUIRES_ERROR);
}

bool g_gadget::startup(int x0,int y0)
{
	calc_sizes();
	calc_placing(x0,y0,SCREEN_W,SCREEN_H);

	keyreset();
	clear_keybuf();

	ext=new extinfo(w,h);

	if(ext->back&&ext->tmp)
	{
		scare_mouse();
		blit(screen,ext->back,x,y,0,0,w,h);
		unscare_mouse();

		if(want_focus())
		{
			get_focus();
			navigate_into(GUINAVIGATE_FORWARD);

			return(true);
		}
	}

	delete ext;
	ext=0;

	return(false);
}

void g_gadget::shutdown()
{
	lose_focus();

	if(ext->back)
	{
		scare_mouse();
		blit(ext->back,screen,0,0,x,y,ext->back->w,ext->back->h);
		unscare_mouse();
	}

	delete ext;
	ext=0;
}

int g_gadget::handle_events()
{
	int k;

	int mx,my,mb;

	if(mouse_needs_poll()) poll_mouse();

	mx=mouse_x;
	my=mouse_y;
	mb=mouse_b;

	ext->event=0;

	if(k=checkallkeys())
	{
		key(k&0xff,k>>8);
	}

	if(!get_flag(GUIFLAG_NOMOUSE))
	{
		if(ext->old_mb!=mb)
		{
			if(mb&~ext->old_mb)
			{
				mouse_click(mx-x,my-y,mb,mb&~ext->old_mb);
			}

			if(ext->old_mb&~mb)
			{
				mouse_release(mx-x,my-y,mb,ext->old_mb&~mb);
			}
		}

		if(ext->old_mx!=mx||ext->old_my!=my)
		{
			mouse_move(mx-x,my-y);
		}

		ext->old_mx=mx;
		ext->old_my=my;
		ext->old_mb=mb;
	}

	if(get_flag(GUIFLAG_REDRAW))
	{
		blit(ext->back,ext->tmp,0,0,0,0,ext->tmp->w,ext->tmp->h);
		draw(ext->tmp,0,0,w,h);
		scare_mouse();
		blit(ext->tmp,screen,0,0,x,y,ext->tmp->w,ext->tmp->h);
		unscare_mouse();

		clear_flag(GUIFLAG_REDRAW);
	}

	return(ext->event);
}

int g_gadget::wait_events()
{
	int res;

	do { res=handle_events(); } while(res==0);

	return(res);
}

void g_gadget::calc_placing(int x0,int y0,int w0,int h0)
{
	if(fixwidth) w=fixwidth;
	else if(w0>maxwidth) w=maxwidth;
	else if(w0<minwidth) w=minwidth; // shouldn't happen!
	else w=w0;

	if(fixheight) h=fixheight;
	else if(h0>maxheight) h=maxheight;
	else if(h0<minheight) h=minheight; // shouldn't happen!
	else h=h0;

	switch(place&GUIPLACE_HORIZ_MASK)
	{
		case GUIPLACE_CENTER: x=x0+(w0-w)/2; break;
		case GUIPLACE_LEFT: x=x0; break;
		case GUIPLACE_RIGHT: x=x0+w0-w; break;
		case GUIPLACE_LEFT_GOLDEN_MEAN: x=x0+(25033*(w0-w))/65536; break;
		case GUIPLACE_RIGHT_GOLDEN_MEAN: x=x0+(40503*(w0-w))/65536; break;
		case GUIPLACE_HORIZ_NONE: break;
	}

	switch(place&GUIPLACE_VERT_MASK)
	{
		case GUIPLACE_MIDDLE: y=y0+(h0-h)/2; break;
		case GUIPLACE_TOP: y=y0; break;
		case GUIPLACE_BOTTOM: y=y0+h0-h; break;
		case GUIPLACE_TOP_GOLDEN_MEAN: y=y0+(25033*(h0-h))/65536; break;
		case GUIPLACE_BOTTOM_GOLDEN_MEAN: y=y0+(40503*(h0-h))/65536; break;
		case GUIPLACE_VERT_NONE: break;
	}

	resize(w,h);
}



g_gadget::extinfo::extinfo()
{
	tmp=back=0;
	event=0;

	old_mx=mouse_x;
	old_my=mouse_y;
	old_mb=0;
}

g_gadget::extinfo::extinfo(int w,int h)
{
	tmp=create_bitmap(w,h);
	back=create_bitmap(w,h);
	event=0;

	old_mx=mouse_x;
	old_my=mouse_y;
	old_mb=0;
}

g_gadget::extinfo::~extinfo()
{
	if(tmp) destroy_bitmap(tmp);
	if(back) destroy_bitmap(back);
}



// g_group

g_group::g_group()
{
	focus=mouseover=0;
}

g_group::~g_group()
{
	children.clear();
}

void g_group::calc_sizes()
{
	g_gadget::calc_sizes();

	iterator i(&children);
	g_gadget *g;
	while(g=(g_gadget *)i.step())
	{
		g->calc_sizes();
	}
}

void g_group::resize(int w,int h)
{
	g_gadget::resize(w,h);

	focus=mouseover=0;
}

void g_group::draw(BITMAP *bm,int x,int y,int w,int h)
{
	g_gadget::draw(bm,x,y,w,h);

	clipregion c(bm);

	iterator i(&children);
	g_gadget *g;
	while(g=(g_gadget *)i.step())
	{
		c.set_extent(x+g->x,y+g->y,g->w,g->h);
		g->draw(bm,x+g->x,y+g->y,g->w,g->h);
	}

	c.restore();
}

bool g_group::key(int c,int k)
{
	if(focus) if(focus->key(c,k)) return(true);

	if(g_gadget::key(c,k)) return(true);

	switch(k)
	{
		case KEY_TAB:
			if(key_shifts&KB_SHIFT_FLAG)
			{
				if(focus_prev(false))
				{
					if(focus) focus->navigate_into(GUINAVIGATE_BACKWARD);
					return(true);
				}
			}
			else
			{
				if(focus_next(false))
				{
					if(focus) focus->navigate_into(GUINAVIGATE_FORWARD);
					return(true);
				}
			}
		break;
	}

	return(false);
}

bool g_group::mouse_click(int x,int y,int b,int ch)
{
	set_focus(find_gadget(x,y));

	if(focus) return(focus->mouse_click(x-focus->x,y-focus->y,b,ch));
	else return(g_gadget::mouse_click(x,y,b,ch));

	return(false);
}

bool g_group::mouse_release(int x,int y,int b,int ch)
{
	bool res=false;

	if(focus) res=focus->mouse_release(x-focus->x,y-focus->y,b,ch);
	else res=g_gadget::mouse_release(x,y,b,ch);

	set_focus(find_gadget(x,y));

	return(res);
}

void g_group::mouse_move(int x,int y)
{
	g_gadget::mouse_move(x,y);

	iterator i(&children);
	g_gadget *g;
	while(g=(g_gadget *)i.step())
	{
		g->mouse_move(x-g->x,y-g->y);
	}

	g=find_gadget(x,y);

	if(g!=mouseover)
	{
		if(mouseover) mouseover->mouse_out(x-mouseover->x,y-mouseover->y);
		if(g) g->mouse_in(x-g->x,y-g->y);
		mouseover=g;
	}

	if(get_flag(GUIFLAG_FOCUSED)) if(!set_focus(g)) set_focus(0);
}

bool g_group::want_focus()
{
	if(!get_flag(GUIFLAG_ACTIVE)) return(false);

	iterator i(&children);
	g_gadget *g;
	while(g=(g_gadget *)i.step())
	{
		if(g->want_focus())
		{
			//focus=g;

			return(true);
		}
	}

	return(g_gadget::want_focus());
}

bool g_group::release_focus()
{
	if(focus) return(focus->release_focus());

	return(g_gadget::release_focus());
}

void g_group::get_focus()
{
	g_gadget::get_focus();

	if(focus) focus->get_focus(); // shouldn't happen
}

void g_group::lose_focus()
{
	g_gadget::lose_focus();

	if(focus) focus->lose_focus();
	focus=0;
}

void g_group::navigate_into(int type)
{
	if(type==GUINAVIGATE_BACKWARD)
	{
		if(set_focus(0)) focus_prev(false);
	}
	else
	{
		if(set_focus(0)) focus_next(false);
	}
}

void g_group::insert_single(g_gadget *gadget)
{
	children.add_tail(gadget);
	gadget->become_child(this);
}

void g_group::insert(g_gadget **gadgets)
{
	while(*gadgets) insert_single(*gadgets++);
}

void g_group::insert(g_gadget *first,...)
{
	insert_va(first);
}

g_gadget *g_group::find_gadget(int x,int y)
{
	iterator i(&children);
	g_gadget *g;
	while(g=(g_gadget *)i.step())
	{
		if(x>=g->x&&y>=g->y&&x<g->x+g->w&&y<g->y+g->h) return(g);
	}
	return(0);
}

bool g_group::set_focus(g_gadget *newfocus)
{
	if(newfocus==focus) return(true);

	if(pressing) return(false);

	if(newfocus)
	if(!newfocus->want_focus()) return(false);

	if(focus)
	{
		if(focus->release_focus())
		{
			focus->lose_focus();
		}
		else return(false);
	}
	if(newfocus) newfocus->get_focus();
	focus=newfocus;

	return(true);
}

bool g_group::focus_next(bool loop)
{
	iterator i;
	g_gadget *g;

	if(focus) { i.start(&children,focus); i.step(); }
	else { i.start(&children); loop=false; }

	while(g=(g_gadget *)i.step())
	{
		if(set_focus(g)) return(true);
	}

	if(loop)
	{
		i.start(&children);
		while(g=(g_gadget *)i.step())
		{
			if(set_focus(g)) return(true);
		}
	}

	return(false);
}

bool g_group::focus_prev(bool loop)
{
	reverseiterator i;
	g_gadget *g;

	if(focus) { i.start(&children,focus); i.step(); }
	else { i.start(&children); loop=false; }

	while(g=(g_gadget *)i.step())
	{
		if(set_focus(g)) return(true);
	}

	if(loop)
	{
		i.start(&children);
		while(g=(g_gadget *)i.step())
		{
			if(set_focus(g)) return(true);
		}
	}

	return(false);
}



// g_gridgroup

g_gridgroup::g_gridgroup(int size,bool fixedrows,bool vertical)
{
	fixrows=fixedrows;
	vert=vertical;

	if(fixedrows) rows=size;
	else cols=size;

	hspace=vspace=2;

	num=0;
	minwidths=minheights=0;
	maxwidths=maxheights=0;
	usewidths=useheights=0;
}

g_gridgroup::~g_gridgroup()
{
	delete minwidths;
	delete minheights;
	delete maxwidths;
	delete maxheights;
	delete usewidths;
	delete useheights;
}

void g_gridgroup::calc_sizes()
{
	g_group::calc_sizes();

	num=children.count();

	if(fixrows) cols=(num+rows-1)/rows;
	else rows=(num+cols-1)/cols;

	delete minwidths;
	delete minheights;
	delete maxwidths;
	delete maxheights;
	delete usewidths;
	delete useheights;

	minwidths=new int[cols];
	minheights=new int[rows];
	maxwidths=new int[cols];
	maxheights=new int[rows];
	usewidths=new int[cols];
	useheights=new int[rows];

	for(int i=0;i<cols;i++) { minwidths[i]=0; maxwidths[i]=GUI_NO_MAX_SIZE; }
	for(int i=0;i<rows;i++) { minheights[i]=0; maxheights[i]=GUI_NO_MAX_SIZE; }

	int n=0,col,row;
	iterator i(&children);
	g_gadget *g;
	while(g=(g_gadget *)i.step())
	{
		if(vert) { col=n/rows; row=n%rows; }
		else { col=n%cols; row=n/cols; }

		minwidths[col]=MAX(minwidths[col],g->fixwidth?g->fixwidth:g->minwidth);
		minheights[row]=MAX(minheights[row],g->fixheight?g->fixheight:g->minheight);

		maxwidths[col]=MIN(maxwidths[col],g->fixwidth?g->fixwidth:g->maxwidth);
		maxheights[row]=MIN(maxheights[row],g->fixheight?g->fixheight:g->maxheight);

		n++;
	}

	for(int i=0;i<cols;i++) if(minwidths[i]>maxwidths[i]) maxwidths[i]=minwidths[i];
	for(int i=0;i<rows;i++) if(minheights[i]>maxheights[i]) maxheights[i]=minheights[i];
	// or should I just adjust maxwidth and maxheight later on?

	minwidth+=(cols-1)*hspace;
	minheight+=(rows-1)*vspace;

	maxwidth+=(cols-1)*hspace;
	maxheight+=(rows-1)*vspace;

	for(int i=0;i<cols;i++) minwidth+=minwidths[i];
	for(int i=0;i<rows;i++) minheight+=minheights[i];

	for(int i=0;i<cols;i++)
	if(maxwidths[i]==GUI_NO_MAX_SIZE) { maxwidth=GUI_NO_MAX_SIZE; break; }
	else maxwidth+=maxwidths[i];

	for(int i=0;i<rows;i++)
	if(maxheights[i]==GUI_NO_MAX_SIZE) { maxheight=GUI_NO_MAX_SIZE; break; }
	else maxheight+=maxheights[i];
}

void g_gridgroup::resize(int w,int h)
{
	g_group::resize(w,h);

	distribute_space(w,h);

	int n=0,x=0,y=0,col,row;
	iterator i(&children);
	g_gadget *g;
	while(g=(g_gadget *)i.step())
	{
		if(vert)
		{
			col=n/rows; row=n%rows;
			if(row==0&&col!=0) { y=0; x+=usewidths[col-1]+hspace; } // starting a new column
		}
		else
		{
			col=n%cols; row=n/cols;
			if(col==0&&row!=0) { x=0; y+=useheights[row-1]+vspace; } // starting a new row
		}

		g->calc_placing(x+frame->left,y+frame->top,usewidths[col],useheights[row]);

		if(vert) y+=useheights[row]+vspace;
		else x+=usewidths[col]+hspace;

		n++;
	}
}

void distribute(int n,int extra_size,int *minsizes,int *maxsizes,int *usesizes)
{
	if(n==1) // special case
	{
		usesizes[0]=minsizes[0]+extra_size;
		// ignores maxsize - should be OK
		return;
	}

	// set sizes to the minima
	for(int i=0;i<n;i++) usesizes[i]=minsizes[i];

	int elements[n],start=0;

	// create an array containing the indexes 0 ... n-1
	for(int i=0;i<n;i++) elements[i]=i;

	// sort elements - insertion sort, could use a better algo.
	for(int i=1;i<n;i++)
	for(int j=i-1;j>=0;j--)
	{
		if(usesizes[elements[i]]>=usesizes[elements[j]])
		{
			int tmp=elements[j+1];
			elements[j+1]=elements[i];
			elements[i]=tmp;
			break;
		}
	}

	for(;;)
	{
		int smallest=usesizes[elements[start]];

		// find next bigger element
		int next=start+1;
		while(next<n&&usesizes[elements[next]]==smallest) next++;

		int delta_size;

		// check to see if no element was found
		if(n==next)
		{
			delta_size=extra_size/(next-start);
		}
		else
		{
			delta_size=usesizes[elements[next]]-smallest;

			// make sure we don't use up too much space
			if(delta_size*(next-start)>extra_size)
			delta_size=extra_size/(next-start);
		}

		// if we are out of space to distribute, stop
		if(delta_size==0) break;

		for(int i=start;i<next;i++)
		{
			// check if the element can receive this much space
			if(usesizes[elements[i]]+delta_size>maxsizes[elements[i]])
			{
				// distribute enough space to reach maxsize
				extra_size-=maxsizes[elements[i]]-usesizes[elements[i]];
				usesizes[elements[i]]=maxsizes[elements[i]];

				// drop element from processing
				elements[i]=elements[start];
				start++;

				// if all sizes are filled up, just exit and disregard
				// the extra space
				if(start==n) return;
			}
			else
			{
				// distribute space normally
				extra_size-=delta_size;
				usesizes[elements[i]]+=delta_size;
			}
		}
	}

	// distribute the last few pixels
	for(int i=start;extra_size&&i<n;i++)
	{
		if(usesizes[elements[i]]<maxsizes[elements[i]])
		{
			extra_size--;
			usesizes[elements[i]]++;
		}
	}
}

void g_gridgroup::distribute_space(int w,int h)
{
/*	int w_extra=w-minwidth;
	int h_extra=h-minheight;
	int w_delta=w_extra/cols;
	int h_delta=h_extra/rows;
	int cols_bigger=w_extra%cols;
	int rows_bigger=w_extra%rows;

	for(int i=0;i<cols;i++)
	{
		usewidths[i]=minwidths[i]+w_delta+(i<cols_bigger?1:0);
	}

	for(int i=0;i<rows;i++)
	{
		useheights[i]=minheights[i]+h_delta+(i<rows_bigger?1:0);
	}*/
	distribute(cols,w-minwidth,minwidths,maxwidths,usewidths);
	distribute(rows,h-minheight,minheights,maxheights,useheights);
}







g_virtual::g_virtual():g_gadget()
{
	x0=y0=0;
	stepsize=16;
}

void g_virtual::calc_sizes()
{
	g_gadget::calc_sizes();

	v_calc_sizes();

	minwidth+=1; // say...
	minheight+=1;

	if(v_maxwidth==GUI_NO_MAX_SIZE) maxwidth=GUI_NO_MAX_SIZE;
	else maxwidth+=v_maxwidth;

	if(v_maxheight==GUI_NO_MAX_SIZE) maxheight=GUI_NO_MAX_SIZE;
	else maxheight+=v_maxheight;
}

void g_virtual::resize(int w,int h)
{
	g_gadget::resize(w,h);

	v_view_width=w-frame->left-frame->right;
	v_view_height=h-frame->top-frame->bottom;

	v_width=v_view_width;
	v_height=v_view_height;

	if(v_width<v_minwidth) v_width=v_minwidth;
	if(v_height<v_minheight) v_height=v_minheight;

	// max should never be exceeded

	v_resize(v_width,v_height);
}

void g_virtual::draw(BITMAP *bm,int x,int y,int w,int h)
{
	g_gadget::draw(bm,x,y,w,h);

	clipregion c(bm);
	c.set_extent(x+frame->left,y+frame->top,
	w-frame->left-frame->right,h-frame->top-frame->bottom);

	v_draw(bm,x-x0+frame->left,y-y0+frame->top,v_width,v_height,
	x0,y0,w-frame->left-frame->right,h-frame->top-frame->bottom);

	c.restore();
}

bool g_virtual::key(int c,int k)
{
	int mult=(key_shifts&KB_SHIFT_FLAG)?8:1;

	switch(k)
	{
		case KEY_LEFT:
			x0-=stepsize*mult;
			if(x0<0) x0=0;
			redraw();
			return(true);

		case KEY_RIGHT:
			x0+=stepsize*mult;
			if(x0>=v_width-v_view_width) x0=v_width-v_view_width;
			redraw();
			return(true);

		case KEY_UP:
			y0-=stepsize*mult;
			if(y0<0) y0=0;
			redraw();
			return(true);

		case KEY_DOWN:
			y0+=stepsize*mult;
			if(y0>=v_height-v_view_height) y0=v_height-v_view_height;
			redraw();
			return(true);
	}
	return(false);
}

bool g_virtual::mouse_click(int x,int y,int b,int ch)
{
	int x_view=x-frame->left,y_view=y-frame->top;

	if(x_view>=0&&y_view>=0&&x_view<v_view_width&&y_view<v_view_height)
	if(v_mouse_click(x_view+x0,y_view+y0,b,ch))
	return(true);

	return(g_gadget::mouse_click(x,y,b,ch));
}

bool g_virtual::mouse_release(int x,int y,int b,int ch)
{
	int x_view=x-frame->left,y_view=y-frame->top;

	if(v_mouse_release(x_view+x0,y_view+y0,b,ch)) return(true);
	return(g_gadget::mouse_release(x,y,b,ch));
}

void g_virtual::mouse_move(int x,int y)
{
	int x_view=x-frame->left,y_view=y-frame->top;

	g_gadget::mouse_move(x,y);
	v_mouse_move(x_view+x0,y_view+y0);
}

void g_virtual::v_calc_sizes() {}

void g_virtual::v_resize(int w,int h) {}

void g_virtual::v_draw(BITMAP *bm,int x,int y,int w,int h,int x0,int y0,int w_vis,int h_vis) {}

bool g_virtual::v_mouse_click(int x,int y,int b,int ch) {}

bool g_virtual::v_mouse_release(int x,int y,int b,int ch) {}

void g_virtual::v_mouse_move(int x,int y) {}



// g_container

g_container::g_container():g_virtual()
{
	g=0;
}

g_container::g_container(g_gadget *content):g_virtual()
{
	g=content;
	g->become_child(this);
}

g_container::~g_container()
{
	delete g;
}

bool g_container::want_focus()
{
	return(g->want_focus());
}

bool g_container::release_focus()
{
	return(g->release_focus());
}

void g_container::get_focus()
{
	g->get_focus();
}

void g_container::lose_focus()
{
	g->lose_focus();
}

void g_container::navigate_into(int type)
{
	g->navigate_into(type);
}

void g_container::v_calc_sizes()
{
	g->calc_sizes();
	v_minwidth=g->minwidth;
	v_minheight=g->minheight;
	v_maxwidth=g->maxwidth;
	v_maxheight=g->maxheight;
}

void g_container::v_resize(int w,int h)
{
	g->resize(w,h);
}

void g_container::v_draw(BITMAP *bm,int x,int y,int w,int h,int x_offs,int y_offs,int w_vis,int h_vis)
{
	g->draw(bm,x,y,w,h);
}

bool g_container::v_mouse_click(int x,int y,int b,int ch)
{
	return(g->mouse_click(x,y,b,ch));
}

bool g_container::v_mouse_release(int x,int y,int b,int ch)
{
	return(g->mouse_release(x,y,b,ch));
}

void g_container::v_mouse_move(int x,int y)
{
	g->mouse_move(x,y);
}

void g_container::set_content(g_gadget *content)
{
	delete g;
	g=content;
	g->become_child(this);
	// do more?
}



// g_text

g_text::g_text(const char *text)
{
	f=font;
	ncol=makecol(0,0,0);
	fcol=makecol(255,255,255);
	pcol=makecol(255,255,255);
	icol=makecol(127,127,127);

	lines=0; numlines=0;
	set_text(text);
}

g_text::~g_text()
{
	set_text(0);
}

void g_text::calc_sizes()
{
	g_gadget::calc_sizes();

	minwidth+=maxline;
	maxwidth=GUI_NO_MAX_SIZE;

	minheight+=numlines*text_height(f);
	maxheight=minheight;
}

void g_text::draw(BITMAP *bm,int x,int y,int w,int h)
{
	g_gadget::draw(bm,x,y,w,h);

	int y_curr=y+frame->top;
	int col;

	if(get_flag(GUIFLAG_PRESSED)) col=pcol;
	else if(get_flag(GUIFLAG_FOCUSED)) col=fcol;
	else if(!get_flag(GUIFLAG_ACTIVE)) col=icol;
	else col=ncol;

	text_mode(-1);

	for(int i=0;i<numlines;i++)
	{
		textout_centre(bm,f,lines[i],
		x+frame->left+(w-frame->left-frame->right)/2,
		y_curr,col);

		y_curr+=text_height(f);
	}
}

void g_text::set_text(const char *text)
{
	for(int i=0;i<numlines;i++) delete lines[i];
	delete lines;

	if(!text)
	{
		numlines=0;
		lines=0;
		maxline=0;
		return;
	}

	numlines=1;
	for(const char *ptr=text;*ptr;ptr++) if(*ptr=='\n') numlines++;
	lines=new char *[numlines];

	char *str=new char[strlen(text)+1];
	strcpy(str,text);

	lines[0]=str;

	int n=1;
	for(char *ptr=str;*ptr;ptr++)
	if(*ptr=='\n')
	{
		*ptr=0;
		lines[n++]=ptr+1;
	}

	maxline=0;
	for(int i=0;i<numlines;i++)
	{
		int len=text_length(f,lines[i]);
		if(len>maxline) maxline=len;
	}
}



// g_button

g_button::g_button(const char *text,int event):g_text(text)
{
	ev=event;

	set_flag(GUIFLAG_CLICKABLE);
}



// g_rectangle

g_rectangle::g_rectangle(int hspacing,int vspacing):g_gadget()
{
	hspace=hspacing;
	vspace=vspacing;
}

void g_rectangle::calc_sizes()
{
	g_gadget::calc_sizes();

	if(hspace)
	{
		minwidth+=hspace;
		maxwidth=minwidth;
	}
	else maxwidth=GUI_NO_MAX_SIZE;

	if(vspace)
	{
		minheight+=vspace;
		maxheight=minheight;
	}
	else maxheight=GUI_NO_MAX_SIZE;
}



// g_spacer

g_spacer::g_spacer(int hspacing,int vspacing):g_rectangle(hspacing,vspacing)
{
	clear_flag(GUIFLAG_ACTIVE);
}





// g_frame

g_frame::g_frame()
{
	left=right=top=bottom=0;
}

void g_frame::draw_normal(BITMAP *bm,int x1,int y1,int x2,int y2) {}
void g_frame::draw_focused(BITMAP *bm,int x1,int y1,int x2,int y2) {}
void g_frame::draw_pressed(BITMAP *bm,int x1,int y1,int x2,int y2) {}
void g_frame::draw_inactive(BITMAP *bm,int x1,int y1,int x2,int y2) {}



// g_noframe

g_noframe::g_noframe():g_frame()
{
	back=makecol(0,92,156);
}

void g_noframe::draw_normal(BITMAP *bm,int x1,int y1,int x2,int y2)
{
	rectfill(bm,x1,y1,x2,y2,back);
}

void g_noframe::draw_focused(BITMAP *bm,int x1,int y1,int x2,int y2)
{
	rectfill(bm,x1,y1,x2,y2,back);
}

void g_noframe::draw_pressed(BITMAP *bm,int x1,int y1,int x2,int y2)
{
	rectfill(bm,x1,y1,x2,y2,back);
}

void g_noframe::draw_inactive(BITMAP *bm,int x1,int y1,int x2,int y2)
{
	rectfill(bm,x1,y1,x2,y2,back);
}



// g_happyframe

g_happyframe::g_happyframe(bool rounded):g_frame()
{
	round=rounded;

	back=makecol(0,92,156);
	shine=makecol(255,255,255);
	shadow=makecol(0,0,0);
	halfshine=makecol(0,120,207);
	halfshadow=makecol(0,61,105);
	highlight=makecol(0,120,207);
//	lowlight=makecol(0,61,105);
//	text=makecol(255,255,255);

	left=right=top=bottom=4;
}

void g_happyframe::draw(BITMAP *bm,int x1,int y1,int x2,int y2,int ol,int il,int b,int ir,int or_)
{
	if(round) draw_round(bm,x1,y1,x2,y2,ol,il,b,ir,or_);
	else draw_square(bm,x1,y1,x2,y2,ol,il,b,ir,or_);
}

void g_happyframe::draw_square(BITMAP *bm,int x1,int y1,int x2,int y2,int ol,int il,int b,int ir,int or_)
{
	// top and bottom
	hline(bm,x1,y1,x2-1,ol);
	hline(bm,x1+1,y1+1,x2-2,il);
	hline(bm,x1+2,y2-1,x2-1,ir);
	hline(bm,x1+1,y2,x2,or_);

	// left and right
	vline(bm,x1,y1+1,y2-1,ol);
	vline(bm,x1+1,y1+2,y2-2,il);
	vline(bm,x2-1,y1+2,y2-2,ir);
	vline(bm,x2,y1+1,y2-1,or_);

	// back
	rectfill(bm,x1+2,y1+2,x2-2,y2-2,b);
}

void g_happyframe::draw_round(BITMAP *bm,int x1,int y1,int x2,int y2,int ol,int il,int b,int ir,int or_)
{
	// top and bottom
	hline(bm,x1+4,y1,x2-4,ol);
	hline(bm,x1+4,y1+1,x2-4,il);
	hline(bm,x1+4,y2-1,x2-4,ir);
	hline(bm,x1+4,y2,x2-4,or_);

	// left and right
	vline(bm,x1,y1+4,y2-4,ol);
	vline(bm,x1+1,y1+4,y2-4,il);
	vline(bm,x2-1,y1+4,y2-4,ir);
	vline(bm,x2,y1+4,y2-4,or_);

	// upper left
	hline(bm,x1+2,y1+1,x1+3,ol);
	hline(bm,x1+2,y1+2,x1+3,il);
	vline(bm,x1+1,y1+2,y1+3,ol);
	putpixel(bm,x1+2,y1+3,il);

	// lower right
	hline(bm,x2-2,y2-1,x2-3,or_);
	hline(bm,x2-2,y2-2,x2-3,ir);
	vline(bm,x2-1,y2-2,y2-3,or_);
	putpixel(bm,x2-2,y2-3,ir);

	// upper right
	hline(bm,x2-2,y1+1,x2-3,ol);
	putpixel(bm,x2-3,y1+2,il);
	vline(bm,x2-1,y1+2,y1+3,or_);
	putpixel(bm,x2-2,y1+3,ir);

	// lower left
	hline(bm,x1+2,y2-1,x1+3,or_);
	putpixel(bm,x1+3,y2-2,ir);
	vline(bm,x1+1,y2-2,y2-3,ol);
	putpixel(bm,x1+2,y2-3,il);

	// back
	hline(bm,x1+4,y1+2,x2-4,b);
	hline(bm,x1+3,y1+3,x2-3,b);
	hline(bm,x1+3,y2-3,x2-3,b);
	hline(bm,x1+4,y2-2,x2-4,b);
	rectfill(bm,x1+2,y1+4,x2-2,y2-4,b);
}



// g_happyframe_button

g_happyframe_button::g_happyframe_button(bool rounded):g_happyframe(rounded)
{
}

void g_happyframe_button::draw_normal(BITMAP *bm,int x1,int y1,int x2,int y2)
{
	draw(bm,x1,y1,x2,y2,shine,halfshine,back,halfshadow,shadow);
}

void g_happyframe_button::draw_focused(BITMAP *bm,int x1,int y1,int x2,int y2)
{
	draw(bm,x1,y1,x2,y2,shine,halfshine,highlight,halfshadow,shadow);
}

void g_happyframe_button::draw_pressed(BITMAP *bm,int x1,int y1,int x2,int y2)
{
	draw(bm,x1,y1,x2,y2,halfshadow,shadow,highlight,shine,halfshine);
}

void g_happyframe_button::draw_inactive(BITMAP *bm,int x1,int y1,int x2,int y2)
{
	draw(bm,x1,y1,x2,y2,halfshine,back,back,back,shadow);
}




// g_happyframe_group

g_happyframe_group::g_happyframe_group(bool rounded):g_happyframe(rounded)
{
}

void g_happyframe_group::draw_normal(BITMAP *bm,int x1,int y1,int x2,int y2)
{
	draw(bm,x1,y1,x2,y2,halfshadow,halfshine,back,halfshadow,halfshine);
}

void g_happyframe_group::draw_focused(BITMAP *bm,int x1,int y1,int x2,int y2)
{
	draw(bm,x1,y1,x2,y2,halfshadow,halfshine,back,halfshadow,halfshine);
}

void g_happyframe_group::draw_pressed(BITMAP *bm,int x1,int y1,int x2,int y2)
{
	draw(bm,x1,y1,x2,y2,halfshine,halfshadow,back,halfshine,halfshadow);
}

void g_happyframe_group::draw_inactive(BITMAP *bm,int x1,int y1,int x2,int y2)
{
	draw(bm,x1,y1,x2,y2,halfshadow,halfshine,back,halfshadow,halfshine);
}



