#include <signal.h>
#include <allegro.h>

#ifdef _OS_WINDOWS_
#include <winalleg.h>
#endif

#include <goa/misc.h>
#include <goa/bitmap.h>
#include <goa/input.h>
#include <goa/types.h>



static void (*crash_func)(char *msg)=0;



void die(char *reason)
{
	if(screen&&crash_func)
	{
		set_clip(screen,0,0,SCREEN_W-1,SCREEN_H-1);
		crash_func(reason);
		//alert("Software failure!","Guru Meditation:",reason,"Die",0,0,0);
	}
	else
	{
		allegro_message("Software failure!\nGuru Meditation: %s\n",reason);
	}

	allegro_exit();

#ifdef _OS_DOS_
	__djgpp_traceback_exit(SIGABRT);
#else
	exit(666);
#endif
}

#ifndef _OS_MACOSX_

static LPTOP_LEVEL_EXCEPTION_FILTER prevExceptionFilter = NULL;

static LONG WINAPI TopLevelExceptionFilter(PEXCEPTION_POINTERS pExceptionInfo)
{
	static BOOL bBeenHere=FALSE;
	
/*	if(!bBeenHere)
	{
		UINT fuOldErrorMode;

		bBeenHere = TRUE;
		
		fuOldErrorMode = SetErrorMode(SEM_FAILCRITICALERRORS | SEM_NOGPFAULTERRORBOX | SEM_NOOPENFILEERRORBOX);

		if(fuOldErrorMode & SEM_NOGPFAULTERRORBOX)
		{
			TCHAR szBuffer[4196];
			
			wsprintf(szBuffer, "An unhandled exception ocurred\r\nSee %s for more details\r\n");
			
			MessageBox(
				NULL, 
				szBuffer,
				"Error",
				MB_OK | MB_ICONERROR
			);
		}
		
		SetErrorMode(fuOldErrorMode);
	}*/
	
	allegro_exit();

//	if(prevExceptionFilter)
//		return prevExceptionFilter(pExceptionInfo);
//	else
		return EXCEPTION_CONTINUE_SEARCH;
}

/*void OnExit(void)
{
	SetUnhandledExceptionFilter(prevExceptionFilter);
}*/

#endif

static void fancy_sighandler(int sig)
{
#ifdef _OS_WINDOWS_
			TCHAR szBuffer[4196];
			
			wsprintf(szBuffer, "blah blah blah\n");
			
			MessageBox(
				NULL, 
				szBuffer,
				"Error",
				MB_OK | MB_ICONERROR
			);

	allegro_exit();
//	raise(sig);
	return;
#endif

	if(screen)
	{
		char *reason;

		switch(sig)
		{
			case SIGSEGV: reason="SIGSEGV"; break;
			case SIGFPE: reason="SIGFPE"; break;
			default: reason="Unknown"; break;
		}

		set_clip(screen,0,0,SCREEN_W-1,SCREEN_H-1);
		crash_func(reason);
	}

	allegro_exit();

#ifdef _OS_DOS_
	__djgpp_traceback_exit(sig);
#else
	exit(666);
#endif
}


void install_fancy_sighandler(void (*crash_function)(char *msg))
{
#ifdef _OS_WINDOWS_
//	if(LoadLibrary("exchndl.dll")) return;
	// if Dr. Mingw DLL is available, use that instead.

	prevExceptionFilter=SetUnhandledExceptionFilter(TopLevelExceptionFilter);
#endif

	if(!crash_function) crash_func=crash_guru;
	else crash_func=crash_function;

	signal(SIGFPE,fancy_sighandler);
	signal(SIGSEGV,fancy_sighandler);
}



int get_cbm_width(uint8 *cbm) { return((cbm[0]<<8)+cbm[1]); }

int get_cbm_height(uint8 *cbm) { return((cbm[2]<<8)+cbm[3]); }

void draw_cbm(BITMAP *bm,uint8 *cbm,int x0,int y0)
{
	int w,h;
	int ncol,col[128];

	w=get_cbm_width(cbm);
	h=get_cbm_height(cbm);
	ncol=cbm[4];

	for(int i=0;i<ncol;i++) col[i]=makecol(cbm[5+i*3],cbm[6+i*3],cbm[7+i*3]);

	int x=0,y=0;
	int8 *ptr=(int8 *)(cbm+5+3*ncol);

	while(y<h)
	{
		int8 val=*ptr++;

		if(val>=0) // single pixel
		{
			putpixel(bm,x+x0,y+y0,col[val]);
			x++;
		}
		else // span
		{
			if(val==-128) val=x-w; // span extends to edge

			int8 val2=*ptr++;

			if(val2>=0) // coloured span
			{
				hline(bm,x+x0,y+y0,x+x0-val-1,col[val2]);
			}
			// else transparent span

			x-=val;
		}

		if(x>=w) { x=0; y++; }
	}
}

extern uint8 panic[];

void crash_geneshaft(char *msg)
{
//	BITMAP *panic=load_any_bitmap(geneshaft_filename);

	solid_mode();
	text_mode(-1);
	show_mouse(0);

	if(panic)
	{
//		draw_sprite(screen,panic,(SCREEN_W-panic->w)/2,(SCREEN_H-panic->h)/2);
		draw_cbm(screen,panic,
		(SCREEN_W-get_cbm_width(panic))/2,
		(SCREEN_H-get_cbm_height(panic))/2);

		int x=SCREEN_W/2;
		int y=(SCREEN_H+get_cbm_height(panic))/2+12;

		textout_centre(screen,font,msg,x-1,y,makecol(0,0,0));
		textout_centre(screen,font,msg,x+1,y,makecol(0,0,0));
		textout_centre(screen,font,msg,x,y-1,makecol(0,0,0));
		textout_centre(screen,font,msg,x,y+1,makecol(0,0,0));
		textout_centre(screen,font,msg,x,y,makecol(255,127,0));

		while(keydown()||mouse_b);
		while(!keydown()&&!mouse_b);
	}
}


#define GURU_HEIGHT 48
#define GURU_THICKNESS 3
#define GURU_SPACE 2

static void draw_guru_frame(int col)
{
	rectfill(screen,0,0,SCREEN_W-1,GURU_THICKNESS-1,col);
	rectfill(screen,0,GURU_HEIGHT-GURU_THICKNESS,SCREEN_W-1,GURU_HEIGHT-1,col);

	rectfill(screen,0,GURU_THICKNESS,GURU_THICKNESS-1,GURU_HEIGHT-GURU_THICKNESS-1,col);
	rectfill(screen,SCREEN_W-GURU_THICKNESS,GURU_THICKNESS,SCREEN_W-1,GURU_HEIGHT-GURU_THICKNESS-1,col);
}

void crash_guru(char *msg)
{
	int red=makecol(255,0,0);
	int black=makecol(0,0,0);

	solid_mode();
	show_mouse(0);

	blit(screen,screen,0,0,0,GURU_HEIGHT+GURU_SPACE,SCREEN_W,SCREEN_H-GURU_HEIGHT-GURU_SPACE);

	rectfill(screen,0,0,SCREEN_W-1,GURU_HEIGHT+GURU_SPACE-1,black);

	textout_centre(screen,font,"Software failure!",SCREEN_W/2,12,red);
	textprintf_centre(screen,font,SCREEN_W/2,28,red,"Guru meditation: %s",msg);

	draw_guru_frame(red);

	bool pressing=true;
	int counter=0;

	while(pressing||!(keydown()||mouse_b))
	{
		if(!keydown()&&!mouse_b) pressing=false;

		rest(10);
		counter=(counter+1)%200;

		if(counter==0) draw_guru_frame(red);
		else if(counter==100) draw_guru_frame(black);
	}
}




uint8 panic[]=
{
	0x01,0xaa,0x00,0x64,0x10,0xfc,0x98,0x1c,0xec,0x8c,0x1c,0xdc,0x84,0x18,0xcc,0x78,
	0x18,0xbc,0x70,0x14,0xac,0x68,0x14,0x9c,0x5c,0x10,0x8c,0x54,0x10,0x7c,0x4c,0x0c,
	0x6c,0x40,0x0c,0x5c,0x38,0x08,0x3c,0x24,0x04,0x2c,0x1c,0x04,0x1c,0x10,0x00,0x0c,
	0x08,0x00,0x00,0x00,0x00,0x81,0xff,0x81,0xff,0xce,0xff,0xec,0x0f,0x9a,0xff,0x81,
	0xff,0x81,0xff,0xce,0xff,0xec,0x0f,0x9a,0xff,0x81,0xff,0x81,0xff,0xce,0xff,0xec,
	0x0f,0x9a,0xff,0x81,0xff,0x81,0xff,0xce,0xff,0xec,0x0f,0x9a,0xff,0x81,0xff,0x81,
	0xff,0xce,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0x9a,0xff,0x81,0xff,0x81,0xff,0xce,
	0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0x9a,0xff,0x81,0xff,0x81,0xff,0xce,0xff,0xfc,
	0x0f,0xf4,0x00,0xfc,0x0f,0x9a,0xff,0x81,0xff,0x81,0xff,0xce,0xff,0xfc,0x0f,0xf4,
	0x00,0xfc,0x0f,0x9a,0xff,0x81,0xff,0x81,0xff,0xce,0xff,0xfc,0x0f,0xf4,0x00,0xfc,
	0x0f,0x9a,0xff,0x81,0xff,0x81,0xff,0xce,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0x9a,
	0xff,0x81,0xff,0x81,0xff,0xce,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0x9a,0xff,0x81,
	0xff,0x81,0xff,0xce,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0x9a,0xff,0x81,0xff,0x81,
	0xff,0xce,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0x9a,0xff,0x81,0xff,0x81,0xff,0xce,
	0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0x9a,0xff,0x81,0xff,0x81,0xff,0xce,0xff,0xfc,
	0x0f,0xf4,0x00,0xfc,0x0f,0x9a,0xff,0x81,0xff,0x81,0xff,0xce,0xff,0xfc,0x0f,0xf4,
	0x00,0xfc,0x0f,0x9a,0xff,0x81,0xff,0x81,0xff,0xce,0xff,0xec,0x0f,0x9a,0xff,0x81,
	0xff,0x81,0xff,0xce,0xff,0xec,0x0f,0x9a,0xff,0x81,0xff,0x81,0xff,0xce,0xff,0xec,
	0x0f,0x9a,0xff,0x81,0xff,0x81,0xff,0xce,0xff,0xec,0x0f,0x9a,0xff,0x81,0xff,0x81,
	0xff,0x81,0xff,0xd3,0xff,0x81,0xff,0x81,0xff,0x81,0xff,0xd3,0xff,0x86,0xff,0xd4,
	0x0f,0xe0,0xff,0xbc,0x0f,0xda,0xff,0xec,0x0f,0xde,0xff,0xbc,0x0f,0x8a,0xff,0xcd,
	0x0f,0xe3,0xff,0xb8,0x0f,0xde,0xff,0xec,0x0f,0xe1,0xff,0xb9,0x0f,0x8c,0xff,0xc8,
	0x0f,0xe6,0xff,0xb6,0x0f,0xe0,0xff,0xec,0x0f,0xe4,0xff,0xb6,0x0f,0x8e,0xff,0xc4,
	0x0f,0xe8,0xff,0xb4,0x0f,0xe2,0xff,0xec,0x0f,0xe6,0xff,0xb4,0x0f,0x90,0xff,0xf7,
	0x0f,0x0d,0x0a,0x07,0x04,0x02,0xdc,0x00,0x02,0x04,0x07,0x0a,0x0d,0xf7,0x0f,0xea,
	0xff,0xfc,0x0f,0xc4,0x00,0x02,0x04,0x07,0x0a,0x0d,0xf7,0x0f,0xe4,0xff,0xfc,0x0f,
	0xf4,0x00,0xfc,0x0f,0xe8,0xff,0xf7,0x0f,0x0d,0x0a,0x07,0x04,0x02,0xc4,0x00,0xfc,
	0x0f,0x91,0xff,0xf9,0x0f,0x0d,0x07,0x02,0xd2,0x00,0x02,0x07,0x0d,0xf9,0x0f,0xeb,
	0xff,0xfc,0x0f,0xbf,0x00,0x02,0x07,0x0d,0xf9,0x0f,0xe5,0xff,0xfc,0x0f,0xf4,0x00,
	0xfc,0x0f,0xe9,0xff,0xf9,0x0f,0x0d,0x07,0x02,0xbf,0x00,0xfc,0x0f,0xbc,0x0f,0xd7,
	0xff,0xf9,0x0f,0x0b,0x04,0xcc,0x00,0x04,0x0b,0xf9,0x0f,0xed,0xff,0xfc,0x0f,0xbc,
	0x00,0x04,0x0b,0xf9,0x0f,0xe7,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xeb,0xff,0xf9,
	0x0f,0x0b,0x04,0xbc,0x00,0xfc,0x0f,0xb9,0x0f,0xdb,0xff,0xfa,0x0f,0x0c,0x04,0xc8,
	0x00,0x04,0x0c,0xfa,0x0f,0xee,0xff,0xfc,0x0f,0xba,0x00,0x04,0x0c,0xfa,0x0f,0xe8,
	0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xec,0xff,0xfa,0x0f,0x0c,0x04,0xba,0x00,0xfc,
	0x0f,0xb6,0x0f,0xdf,0xff,0xfa,0x0f,0x08,0xc4,0x00,0x08,0xfa,0x0f,0xef,0xff,0xfc,
	0x0f,0xb8,0x00,0x08,0xfa,0x0f,0xe9,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xed,0xff,
	0xfa,0x0f,0x08,0xb8,0x00,0xfc,0x0f,0xb4,0x0f,0xe2,0xff,0xfb,0x0f,0x0d,0x03,0xc2,
	0x00,0x03,0x0d,0xfb,0x0f,0xf0,0xff,0xfc,0x0f,0xb7,0x00,0x03,0x0d,0xfb,0x0f,0xea,
	0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xee,0xff,0xfb,0x0f,0x0e,0x03,0xb7,0x00,0xfc,
	0x0f,0xfc,0x0f,0xc4,0x00,0x02,0x04,0x07,0x0a,0x0d,0xf7,0x0f,0xe5,0xff,0xfb,0x0f,
	0x0c,0x01,0xc0,0x00,0x01,0x0c,0xfb,0x0f,0xf1,0xff,0xfc,0x0f,0xb6,0x00,0x01,0x0c,
	0xfb,0x0f,0xeb,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xef,0xff,0xfb,0x0f,0x0c,0x01,
	0xb6,0x00,0xfc,0x0f,0xfc,0x0f,0xbf,0x00,0x02,0x07,0x0d,0xf9,0x0f,0xe7,0xff,0xfb,
	0x0f,0x0c,0x01,0xbe,0x00,0x01,0x0c,0xfb,0x0f,0xf2,0xff,0xfc,0x0f,0xb5,0x00,0x01,
	0x0c,0xfb,0x0f,0xec,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf0,0xff,0xfb,0x0f,0x0c,
	0x01,0xb5,0x00,0xfc,0x0f,0xfc,0x0f,0xbc,0x00,0x04,0x0b,0xf9,0x0f,0xe9,0xff,0xfc,
	0x0f,0x0c,0x01,0xbc,0x00,0x01,0x0c,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xb4,0x00,0x01,
	0x0c,0xfc,0x0f,0xec,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf0,0xff,0xfc,0x0f,0x0c,
	0x01,0xb4,0x00,0xfc,0x0f,0xfc,0x0f,0xba,0x00,0x04,0x0c,0xfa,0x0f,0xeb,0xff,0xfc,
	0x0f,0x0d,0x01,0xba,0x00,0x01,0x0d,0xfc,0x0f,0xf3,0xff,0xfc,0x0f,0xb3,0x00,0x01,
	0x0d,0xfc,0x0f,0xed,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf1,0xff,0xfc,0x0f,0x0e,
	0x01,0xb3,0x00,0xfc,0x0f,0xfc,0x0f,0xb8,0x00,0x08,0xfa,0x0f,0xed,0xff,0xfb,0x0f,
	0x03,0xb8,0x00,0x03,0xfb,0x0f,0xf4,0xff,0xfc,0x0f,0xb2,0x00,0x03,0xfb,0x0f,0xee,
	0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,0xfb,0x0f,0x03,0xb2,0x00,0xfc,0x0f,
	0xfc,0x0f,0xb7,0x00,0x03,0x0d,0xfb,0x0f,0xee,0xff,0xfc,0x0f,0x08,0xb6,0x00,0x08,
	0xfc,0x0f,0xf4,0xff,0xfc,0x0f,0xb1,0x00,0x08,0xfc,0x0f,0xee,0xff,0xfc,0x0f,0xf4,
	0x00,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0x08,0xb1,0x00,0xfc,0x0f,0xfc,0x0f,0xb6,0x00,
	0x01,0x0c,0xfb,0x0f,0xf0,0xff,0xfc,0x0f,0x0c,0xef,0x00,0x05,0x09,0x0b,0xdc,0x0f,
	0x0b,0x09,0x05,0xef,0x00,0x0c,0xfc,0x0f,0xf5,0xff,0xfc,0x0f,0xf4,0x00,0xd0,0x0f,
	0x0b,0x09,0x05,0xef,0x00,0x0c,0xfc,0x0f,0xef,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xf3,0xff,0xfc,0x0f,0x0c,0xef,0x00,0x05,0x09,0x0b,0xc0,0x0f,0xfc,0x0f,0xb5,0x00,
	0x01,0x0c,0xfb,0x0f,0xf1,0xff,0xfc,0x0f,0x04,0xf1,0x00,0x08,0x0d,0xd6,0x0f,0x0d,
	0x08,0xf1,0x00,0x04,0xfc,0x0f,0xf5,0xff,0xfc,0x0f,0xf4,0x00,0xcd,0x0f,0x0d,0x08,
	0xf1,0x00,0x04,0xfc,0x0f,0xef,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf3,0xff,0xfc,
	0x0f,0x04,0xf1,0x00,0x07,0x0d,0xbd,0x0f,0xfc,0x0f,0xb4,0x00,0x01,0x0c,0xfc,0x0f,
	0xf2,0xff,0xfc,0x0f,0x0b,0xf2,0x00,0x03,0x0d,0xd2,0x0f,0x0d,0x03,0xf2,0x00,0x0b,
	0xfc,0x0f,0xf6,0xff,0xfc,0x0f,0xf4,0x00,0xcb,0x0f,0x0d,0x03,0xf2,0x00,0x0b,0xfc,
	0x0f,0xf0,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf4,0xff,0xfc,0x0f,0x0b,0xf2,0x00,
	0x03,0x0c,0xbb,0x0f,0xfc,0x0f,0xb3,0x00,0x01,0x0d,0xfc,0x0f,0xf3,0xff,0xfc,0x0f,
	0x04,0xf3,0x00,0x06,0xce,0x0f,0x06,0xf3,0x00,0x04,0xfc,0x0f,0xf6,0xff,0xfc,0x0f,
	0xf4,0x00,0xc9,0x0f,0x06,0xf3,0x00,0x04,0xfc,0x0f,0xf0,0xff,0xfc,0x0f,0xf4,0x00,
	0xfc,0x0f,0xf4,0xff,0xfc,0x0f,0x04,0xf3,0x00,0x06,0xb9,0x0f,0xfc,0x0f,0xb2,0x00,
	0x03,0xfb,0x0f,0xf4,0xff,0xfd,0x0f,0x0c,0xf3,0x00,0x06,0xf9,0x0f,0xda,0xff,0xf9,
	0x0f,0x06,0xf3,0x00,0x0d,0xfd,0x0f,0xf6,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xd3,
	0xff,0xf9,0x0f,0x06,0xf3,0x00,0x0c,0xfc,0x0f,0xf1,0xff,0xfc,0x0f,0xf4,0x00,0xfc,
	0x0f,0xf4,0xff,0xfd,0x0f,0x0d,0xf3,0x00,0x06,0xf9,0x0f,0xbf,0xff,0xfc,0x0f,0xb1,
	0x00,0x08,0xfc,0x0f,0xf5,0xff,0xfc,0x0f,0x07,0xf4,0x00,0x03,0xfa,0x0f,0xd6,0xff,
	0xfa,0x0f,0x03,0xf4,0x00,0x07,0xfc,0x0f,0xf7,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xd1,0xff,0xfa,0x0f,0x03,0xf4,0x00,0x07,0xfc,0x0f,0xf1,0xff,0xfc,0x0f,0xf4,0x00,
	0xfc,0x0f,0xf5,0xff,0xfc,0x0f,0x07,0xf4,0x00,0x03,0xfa,0x0f,0xbd,0xff,0xfc,0x0f,
	0xf4,0x00,0xd0,0x0f,0x0b,0x09,0x05,0xef,0x00,0x0c,0xfc,0x0f,0xf6,0xff,0xfc,0x0f,
	0x02,0xf4,0x00,0x0d,0xfc,0x0f,0xd2,0xff,0xfc,0x0f,0x0d,0xf4,0x00,0x02,0xfc,0x0f,
	0xf7,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcf,0xff,0xfc,0x0f,0x0d,0xf4,0x00,0x02,
	0xfc,0x0f,0xf1,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf5,0xff,0xfc,0x0f,0x02,0xf4,
	0x00,0x0c,0xfb,0x0f,0xbc,0xff,0xfc,0x0f,0xf4,0x00,0xcd,0x0f,0x0d,0x08,0xf1,0x00,
	0x04,0xfc,0x0f,0xf6,0xff,0xfd,0x0f,0x0d,0xf4,0x00,0x08,0xfc,0x0f,0xd0,0xff,0xfc,
	0x0f,0x08,0xf4,0x00,0x0d,0xfd,0x0f,0xf7,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xce,
	0xff,0xfc,0x0f,0x08,0xf4,0x00,0x0c,0xfd,0x0f,0xf1,0xff,0xfc,0x0f,0xf4,0x00,0xfc,
	0x0f,0xf5,0xff,0xfd,0x0f,0x0d,0xf4,0x00,0x06,0xfb,0x0f,0xbb,0xff,0xfc,0x0f,0xf4,
	0x00,0xcb,0x0f,0x0d,0x03,0xf2,0x00,0x0b,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0x0a,0xf4,
	0x00,0x0d,0xfc,0x0f,0xd0,0xff,0xfc,0x0f,0x0d,0xf4,0x00,0x0a,0xfc,0x0f,0xf8,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xce,0xff,0xfc,0x0f,0x0d,0xf4,0x00,0x0a,0xfc,0x0f,
	0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf6,0xff,0xfc,0x0f,0x0a,0xf4,0x00,0x0d,
	0xfc,0x0f,0xba,0xff,0xfc,0x0f,0xf4,0x00,0xc9,0x0f,0x06,0xf3,0x00,0x04,0xfc,0x0f,
	0xf8,0xff,0xfc,0x0f,0x07,0xf5,0x00,0x05,0xfc,0x0f,0xce,0xff,0xfc,0x0f,0x05,0xf5,
	0x00,0x07,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcd,0xff,0xfc,0x0f,
	0x05,0xf5,0x00,0x07,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf6,0xff,
	0xfc,0x0f,0x07,0xf5,0x00,0x05,0xfc,0x0f,0xb9,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xd3,0xff,0xf9,0x0f,0x06,0xf3,0x00,0x0d,0xfd,0x0f,0xf8,0xff,0xfc,0x0f,0x04,0xf5,
	0x00,0x09,0xfc,0x0f,0xce,0xff,0xfc,0x0f,0x09,0xf5,0x00,0x04,0xfc,0x0f,0xf8,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcd,0xff,0xfc,0x0f,0x09,0xf5,0x00,0x04,0xfc,0x0f,
	0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf6,0xff,0xfc,0x0f,0x04,0xf5,0x00,0x09,
	0xfc,0x0f,0xb9,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xd1,0xff,0xfa,0x0f,0x03,0xf4,
	0x00,0x07,0xfc,0x0f,0xf9,0xff,0xfc,0x0f,0x02,0xf5,0x00,0x0b,0xfd,0x0f,0xcc,0xff,
	0xfd,0x0f,0x0b,0xf5,0x00,0x02,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xcc,0xff,0xfd,0x0f,0x0b,0xf5,0x00,0x02,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xf4,0x00,
	0xfc,0x0f,0xf6,0xff,0xfc,0x0f,0x03,0xf5,0x00,0x0b,0xfd,0x0f,0xb8,0xff,0xfc,0x0f,
	0xf4,0x00,0xfc,0x0f,0xcf,0xff,0xfc,0x0f,0x0d,0xf4,0x00,0x02,0xfc,0x0f,0xf9,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf6,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xb8,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xce,0xff,0xfc,0x0f,0x08,0xf4,0x00,0x0d,0xfd,0x0f,
	0xf9,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf6,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xb8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xce,0xff,0xfc,0x0f,0x0d,0xf4,0x00,0x0a,
	0xfc,0x0f,0xfa,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,
	0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,
	0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf6,0xff,0xfc,0x0f,0xf4,0x00,
	0xfc,0x0f,0xb8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcd,0xff,0xfc,0x0f,0x05,0xf5,
	0x00,0x07,0xfc,0x0f,0xfa,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,
	0xf4,0x00,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,
	0xf4,0x00,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf6,0xff,0xfc,0x0f,
	0xf4,0x00,0xfc,0x0f,0xb8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcd,0xff,0xfc,0x0f,
	0x09,0xf5,0x00,0x04,0xfc,0x0f,0xfa,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf6,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xb8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,
	0xfd,0x0f,0x0b,0xf5,0x00,0x02,0xfc,0x0f,0xfa,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xf6,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xb8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xfa,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xf6,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xb8,0xff,0xfc,0x0f,0xf5,0x00,0x01,0xfc,
	0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xfa,0xff,0xfc,0x0f,0xf4,0x00,0xfc,
	0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,
	0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,
	0x0f,0xf6,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xb8,0xff,0xfc,0x0f,0xf5,0x00,0x04,
	0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xfa,0xff,0xfc,0x0f,0xf4,0x00,
	0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,
	0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xf4,0x00,
	0xfc,0x0f,0xf6,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xb8,0xff,0xfc,0x0f,0xf5,0x00,
	0x08,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xfa,0xff,0xfc,0x0f,0xf4,
	0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,
	0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xf4,
	0x00,0xfc,0x0f,0xf6,0xff,0xfc,0x0f,0xf4,0x00,0x0e,0xfd,0x0f,0xb8,0xff,0xfc,0x0f,
	0xf5,0x00,0x0c,0xfd,0x0f,0xcb,0xff,0xfd,0x0f,0x0b,0xf5,0x00,0x02,0xfc,0x0f,0xfa,
	0xff,0xfc,0x0f,0x02,0xf5,0x00,0x0b,0xfd,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,
	0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,
	0x0f,0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf6,0xff,0xfc,0x0f,0x03,0xf5,0x00,
	0x0b,0xfd,0x0f,0xb8,0xff,0xfc,0x0f,0xf6,0x00,0x02,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,
	0x09,0xf5,0x00,0x04,0xfc,0x0f,0xfa,0xff,0xfc,0x0f,0x04,0xf5,0x00,0x09,0xfc,0x0f,
	0xcd,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xf6,0xff,0xfc,0x0f,0x04,0xf5,0x00,0x09,0xfc,0x0f,0xb9,0xff,0xfc,0x0f,0xf6,0x00,
	0x0a,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0x05,0xf5,0x00,0x07,0xfc,0x0f,0xfa,0xff,0xfc,
	0x0f,0x07,0xf5,0x00,0x05,0xfc,0x0f,0xcd,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,
	0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,
	0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf6,0xff,0xfc,0x0f,0x07,0xf5,0x00,0x05,0xfc,
	0x0f,0xb9,0xff,0xfc,0x0f,0xf7,0x00,0x04,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0x0d,0xf4,
	0x00,0x0a,0xfc,0x0f,0xfa,0xff,0xfc,0x0f,0x0a,0xf4,0x00,0x0d,0xfc,0x0f,0xce,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf6,0xff,
	0xfc,0x0f,0x0a,0xf4,0x00,0x0d,0xfc,0x0f,0xba,0xff,0xfc,0x0f,0xf8,0x00,0x01,0x0d,
	0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0x08,0xf4,0x00,0x0d,0xfd,0x0f,0xf8,0xff,0xfd,0x0f,
	0x0d,0xf4,0x00,0x08,0xfc,0x0f,0xce,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf5,0xff,0xfd,0x0f,0x0d,0xf4,0x00,0x06,0xfb,0x0f,
	0xbb,0xff,0xfc,0x0f,0xf9,0x00,0x01,0x0c,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0x0d,0xf4,
	0x00,0x02,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0x02,0xf4,0x00,0x0d,0xfc,0x0f,0xcf,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf5,0xff,
	0xfc,0x0f,0x02,0xf4,0x00,0x0c,0xfb,0x0f,0xbc,0xff,0xfc,0x0f,0xfa,0x00,0x01,0x0c,
	0xfb,0x0f,0xce,0xff,0xfa,0x0f,0x03,0xf4,0x00,0x07,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,
	0x07,0xf4,0x00,0x03,0xfa,0x0f,0xd1,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf5,0xff,0xfc,0x0f,0x07,0xf4,0x00,0x03,0xfa,0x0f,
	0xbd,0xff,0xfc,0x0f,0xfb,0x00,0x04,0x0d,0xfb,0x0f,0xcf,0xff,0xf9,0x0f,0x06,0xf3,
	0x00,0x0d,0xfd,0x0f,0xf6,0xff,0xfd,0x0f,0x0d,0xf3,0x00,0x06,0xf9,0x0f,0xd3,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf4,0xff,
	0xfd,0x0f,0x0d,0xf3,0x00,0x06,0xf9,0x0f,0xbf,0xff,0xfc,0x0f,0xfd,0x00,0x02,0x0a,
	0xfa,0x0f,0xf7,0xff,0xd1,0x0f,0x06,0xf3,0x00,0x04,0xfc,0x0f,0xf6,0xff,0xfc,0x0f,
	0x04,0xf3,0x00,0x06,0xe1,0x0f,0xec,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf4,0xff,0xfc,0x0f,0x04,0xf3,0x00,0x06,0xb9,0x0f,
	0xfc,0x0f,0x04,0x08,0x0c,0xf9,0x0f,0xfa,0xff,0xcf,0x0f,0x0d,0x03,0xf2,0x00,0x0b,
	0xfc,0x0f,0xf6,0xff,0xfc,0x0f,0x0b,0xf2,0x00,0x03,0x0d,0xe0,0x0f,0xef,0xff,0xfc,
	0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,
	0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf4,0xff,0xfc,
	0x0f,0x0b,0xf2,0x00,0x03,0x0c,0xbb,0x0f,0xf3,0x0f,0xfb,0xff,0xcf,0x0f,0x0d,0x08,
	0xf1,0x00,0x04,0xfc,0x0f,0xf4,0xff,0xfc,0x0f,0x04,0xf1,0x00,0x08,0x0d,0xe0,0x0f,
	0xf1,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xf3,0xff,0xfc,0x0f,0x04,0xf1,0x00,0x08,0x0d,0xbd,0x0f,0xf5,0x0f,0xfb,0xff,0xd0,
	0x0f,0x0b,0x09,0x05,0xef,0x00,0x0c,0xfc,0x0f,0xf4,0xff,0xfc,0x0f,0x0c,0xef,0x00,
	0x05,0x09,0x0b,0xe1,0x0f,0xf3,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,0xfc,
	0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,0xfc,
	0x0f,0xf4,0x00,0xfc,0x0f,0xf3,0xff,0xfc,0x0f,0x0c,0xef,0x00,0x05,0x09,0x0b,0xc0,
	0x0f,0xf7,0x0f,0xfb,0xff,0xf7,0x0f,0x0c,0x0a,0x07,0x04,0x02,0xc9,0x00,0x08,0xfc,
	0x0f,0xf2,0xff,0xfc,0x0f,0x08,0xd8,0x00,0x01,0x04,0x07,0x0c,0xf9,0x0f,0xf4,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,
	0xfc,0x0f,0x08,0xb1,0x00,0xfc,0x0f,0xfa,0x0f,0xf9,0xff,0xf9,0x0f,0x0c,0x07,0x02,
	0xc5,0x00,0x03,0xfb,0x0f,0xf2,0xff,0xfb,0x0f,0x03,0xd5,0x00,0x02,0x0a,0xfa,0x0f,
	0xf5,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xf2,0xff,0xfb,0x0f,0x03,0xb2,0x00,0xfc,0x0f,0xf5,0xff,0xf9,0x0f,0x0b,0x04,0xc3,
	0x00,0x01,0x0d,0xfc,0x0f,0xf0,0xff,0xfc,0x0f,0x0d,0x01,0xd4,0x00,0x04,0x0d,0xfb,
	0x0f,0xf6,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,
	0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,
	0x0f,0xf1,0xff,0xfc,0x0f,0x0e,0x01,0xb3,0x00,0xfc,0x0f,0xf6,0xff,0xfa,0x0f,0x0c,
	0x04,0xc2,0x00,0x01,0x0c,0xfc,0x0f,0xee,0xff,0xfc,0x0f,0x0c,0x01,0xd4,0x00,0x01,
	0x0c,0xfb,0x0f,0xf7,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,
	0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xf4,
	0x00,0xfc,0x0f,0xf0,0xff,0xfc,0x0f,0x0c,0x01,0xb4,0x00,0xfc,0x0f,0xf7,0xff,0xfa,
	0x0f,0x08,0xc1,0x00,0x01,0x0c,0xfb,0x0f,0xee,0xff,0xfb,0x0f,0x0c,0x01,0xd4,0x00,
	0x01,0x0c,0xfc,0x0f,0xf7,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,
	0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,
	0xf4,0x00,0xfc,0x0f,0xf0,0xff,0xfb,0x0f,0x0c,0x01,0xb5,0x00,0xfc,0x0f,0xf8,0xff,
	0xfb,0x0f,0x0d,0x03,0xc1,0x00,0x01,0x0c,0xfb,0x0f,0xec,0xff,0xfb,0x0f,0x0c,0x01,
	0xd4,0x00,0x01,0x0d,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xef,0xff,0xfb,0x0f,0x0c,0x01,0xb6,0x00,0xfc,0x0f,
	0xf9,0xff,0xfb,0x0f,0x0c,0x01,0xc1,0x00,0x03,0x0d,0xfb,0x0f,0xea,0xff,0xfb,0x0f,
	0x0d,0x03,0xd4,0x00,0x04,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,
	0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,
	0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xee,0xff,0xfb,0x0f,0x0e,0x03,0xb7,0x00,0xfc,
	0x0f,0xfa,0xff,0xfb,0x0f,0x0c,0x01,0xc1,0x00,0x08,0xfa,0x0f,0xe8,0xff,0xfa,0x0f,
	0x08,0xd4,0x00,0x0a,0xfc,0x0f,0xf9,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xed,0xff,0xfa,0x0f,0x08,0xb8,0x00,0xfc,0x0f,0xfa,
	0xff,0xfc,0x0f,0x0c,0x01,0xc2,0x00,0x04,0x0c,0xfa,0x0f,0xe6,0xff,0xfa,0x0f,0x0c,
	0x04,0xd6,0x00,0x02,0xfc,0x0f,0xf9,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,
	0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xec,0xff,0xfa,0x0f,0x0c,0x04,0xba,0x00,0xfc,0x0f,
	0xfb,0xff,0xfc,0x0f,0x0d,0x01,0xc3,0x00,0x04,0x0b,0xf9,0x0f,0xe4,0xff,0xf9,0x0f,
	0x0b,0x04,0xd7,0x00,0x0c,0xfd,0x0f,0xf9,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,
	0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,
	0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xeb,0xff,0xf9,0x0f,0x0b,0x04,0xbc,0x00,0xfc,
	0x0f,0xfc,0xff,0xfb,0x0f,0x03,0xc5,0x00,0x02,0x07,0x0d,0xf9,0x0f,0xe0,0xff,0xf9,
	0x0f,0x0d,0x07,0x02,0xda,0x00,0x07,0xfc,0x0f,0xfa,0xff,0xfc,0x0f,0xf4,0x00,0xfc,
	0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,
	0x0f,0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xe9,0xff,0xf9,0x0f,0x0d,0x07,0x02,
	0xbf,0x00,0xfc,0x0f,0xfc,0xff,0xfc,0x0f,0x08,0xc9,0x00,0x02,0x04,0x07,0x0a,0x0d,
	0xf7,0x0f,0xde,0xff,0xf7,0x0f,0x0d,0x0a,0x07,0x04,0x02,0xdf,0x00,0x04,0xfc,0x0f,
	0xfa,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf8,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xcc,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,0xf2,0xff,0xfc,0x0f,0xf4,0x00,0xfc,0x0f,
	0xe8,0xff,0xf7,0x0f,0x0d,0x0a,0x07,0x04,0x02,0xc4,0x00,0xfc,0x0f,0xfd,0xff,0xfc,
	0x0f,0x0c,0xef,0x00,0x05,0x09,0x0b,0xd0,0x0f,0xda,0xff,0xce,0x0f,0xfa,0xff,0xec,
	0x0f,0xf8,0xff,0xec,0x0f,0xcc,0xff,0xec,0x0f,0xf2,0xff,0xec,0x0f,0xe6,0xff,0xb4,
	0x0f,0xfd,0xff,0xfc,0x0f,0x04,0xf1,0x00,0x08,0x0d,0xcf,0x0f,0xd6,0xff,0xd0,0x0f,
	0xfa,0xff,0xec,0x0f,0xf8,0xff,0xec,0x0f,0xcc,0xff,0xec,0x0f,0xf2,0xff,0xec,0x0f,
	0xe4,0xff,0xb6,0x0f,0xfe,0xff,0xfc,0x0f,0x0b,0xf2,0x00,0x03,0x0d,0xd0,0x0f,0xd0,
	0xff,0xd3,0x0f,0xfa,0xff,0xec,0x0f,0xf8,0xff,0xec,0x0f,0xcc,0xff,0xec,0x0f,0xf2,
	0xff,0xec,0x0f,0xe1,0xff,0xb9,0x0f,0xfe,0xff,0xfc,0x0f,0x04,0xf3,0x00,0x06,0xd1,
	0x0f,0xca,0xff,0xd6,0x0f,0xfa,0xff,0xec,0x0f,0xf8,0xff,0xec,0x0f,0xcc,0xff,0xec,
	0x0f,0xf2,0xff,0xec,0x0f,0xde,0xff,0xbc,0x0f,0xfe,0xff,0xfd,0x0f,0x0c,0xf3,0x00,
	0x06,0xf9,0x0f,0x81,0xff,0x81,0xff,0x81,0xff,0xee,0xff,0xff,0xff,0xfc,0x0f,0x07,
	0xf4,0x00,0x03,0xfa,0x0f,0x81,0xff,0x81,0xff,0x81,0xff,0xec,0xff,0xff,0xff,0xfc,
	0x0f,0x02,0xf4,0x00,0x0d,0xfc,0x0f,0x81,0xff,0x81,0xff,0x81,0xff,0xea,0xff,0xff,
	0xff,0xfd,0x0f,0x0d,0xf4,0x00,0x08,0xfc,0x0f,0x81,0xff,0x81,0xff,0x81,0xff,0xe9,
	0xff,0xfc,0x0f,0x0a,0xf4,0x00,0x0d,0xfc,0x0f,0x81,0xff,0x81,0xff,0x81,0xff,0xe9,
	0xff,0xfc,0x0f,0x07,0xf5,0x00,0x05,0xfc,0x0f,0x81,0xff,0x81,0xff,0x81,0xff,0xe8,
	0xff,0xfc,0x0f,0x04,0xf5,0x00,0x09,0xfc,0x0f,0x81,0xff,0x81,0xff,0x81,0xff,0xe8,
	0xff,0xfc,0x0f,0x02,0xf5,0x00,0x0b,0xfd,0x0f,0x81,0xff,0x81,0xff,0x81,0xff,0xe7,
	0xff,0xec,0x0f,0x81,0xff,0x81,0xff,0x81,0xff,0xe7,0xff,0xec,0x0f,0x81,0xff,0x81,
	0xff,0x81,0xff,0xe7,0xff,0xec,0x0f,0x81,0xff,0x81,0xff,0x81,0xff,0xe7,0xff,0xec,
	0x0f,0x81,0xff,0x81,0xff,0x81,0xff,0xe7,0xff,
};
