#include <allegro.h>

#include <goa/rle.h>
#include <goa/line.h>

#include <goa/rle_inline.h>
//#include "line_inline.h"



bool collide_rle_sprite(RLE_SPRITE *rle1,RLE_SPRITE *rle2,int x1,int y1,int x2,int y2)
{
	int h=MIN(y1+rle1->h,y2+rle2->h)-MAX(y1,y2);

	if(h<=0) return(false); // y bounding box

	void *ptr1,*ptr2;
	int dx,dy;

	if(y1<y2)
	{
		ptr1=rle1->dat; ptr2=rle2->dat;
		dx=x2-x1;
		if(dx<=-rle2->w||dx>=rle1->w) return(false); // x bounding box
		dy=y2-y1;
	}
	else
	{
		ptr1=rle2->dat; ptr2=rle1->dat;
		dx=x1-x2;
		if(dx<=-rle1->w||dx>=rle2->w) return(false); // x bounding box
		dy=y1-y2;
	}

	switch(rle1->color_depth)
	{
		case 8:
		{
			rledat8 *dat1=(rledat8 *)ptr1,*dat2=(rledat8 *)ptr2;
			_rle_wind8(dat1,dy);

			while(h--)
			{
				int x=dx;

				_rle_find_solid_rev8(dat1,x);
				_rle_find_solid8(dat2,x);

				while(*dat1!=MASK_COLOR_8&&*dat2!=MASK_COLOR_8)
				if(x>0)
				{
					x-=*dat1; dat1+=*dat1+1;
					if(x<0) return(true);
					_rle_find_solid_rev8(dat1,x);
				}
				else
				{
					x+=*dat2; dat2+=*dat2+1;
					if(x>0) return(true);
					_rle_find_solid8(dat2,x);
				}

				_rle_next_line8(dat1);
				_rle_next_line8(dat2);
			}
		}
		break;

		case 15:
		{
			rledat15 *dat1=(rledat15 *)ptr1,*dat2=(rledat15 *)ptr2;
			_rle_wind15(dat1,dy);

			while(h--)
			{
				int x=dx;

				_rle_find_solid_rev15(dat1,x);
				_rle_find_solid15(dat2,x);

				while(*dat1!=MASK_COLOR_15&&*dat2!=MASK_COLOR_15)
				if(x>0)
				{
					x-=*dat1; dat1+=*dat1+1;
					if(x<0) return(true);
					_rle_find_solid_rev15(dat1,x);
				}
				else
				{
					x+=*dat2; dat2+=*dat2+1;
					if(x>0) return(true);
					_rle_find_solid15(dat2,x);
				}

				_rle_next_line15(dat1);
				_rle_next_line15(dat2);
			}
		}
		break;

		case 16:
		{
			rledat16 *dat1=(rledat16 *)ptr1,*dat2=(rledat16 *)ptr2;
			_rle_wind16(dat1,dy);

			while(h--)
			{
				int x=dx;

				_rle_find_solid_rev16(dat1,x);
				_rle_find_solid16(dat2,x);

				while((unsigned short)*dat1!=MASK_COLOR_16&&(unsigned short)*dat2!=MASK_COLOR_16)
				if(x>0)
				{
					x-=*dat1; dat1+=*dat1+1;
					if(x<0) return(true);
					_rle_find_solid_rev16(dat1,x);
				}
				else
				{
					x+=*dat2; dat2+=*dat2+1;
					if(x>0) return(true);
					_rle_find_solid16(dat2,x);
				}

				_rle_next_line16(dat1);
				_rle_next_line16(dat2);
			}
		}
		break;

		case 24:
		case 32:
		{
			rledat24 *dat1=(rledat24 *)ptr1,*dat2=(rledat24 *)ptr2;
			_rle_wind24(dat1,dy);

			while(h--)
			{
				int x=dx;

				_rle_find_solid_rev24(dat1,x);
				_rle_find_solid24(dat2,x);

				while(*dat1!=MASK_COLOR_24&&*dat2!=MASK_COLOR_24)
				if(x>0)
				{
					x-=*dat1; dat1+=*dat1+1;
					if(x<0) return(true);
					_rle_find_solid_rev24(dat1,x);
				}
				else
				{
					x+=*dat2; dat2+=*dat2+1;
					if(x>0) return(true);
					_rle_find_solid24(dat2,x);
				}

				_rle_next_line24(dat1);
				_rle_next_line24(dat2);
			}
		}
		break;
	}

	return(false);
}

bool collide_rle_point(RLE_SPRITE *rle,int x1,int y1,int x2,int y2)
{
	int y=y2-y1,x=x2-x1;

	if(y<0||y>=rle->h||x<0||x>=rle->w) return(false); // clip

	switch(rle->color_depth)
	{
		case 8:
		{
			rledat8 *dat=(rledat8 *)rle->dat;
			_rle_wind8(dat,y);
			return(_rle_collide_point8(dat,x));
		}
		break;

		case 15:
		{
			rledat15 *dat=(rledat15 *)rle->dat;
			_rle_wind15(dat,y);
			return(_rle_collide_point15(dat,x));
		}
		break;

		case 16:
		{
			rledat16 *dat=(rledat16 *)rle->dat;
			_rle_wind16(dat,y);
			return(_rle_collide_point16(dat,x));
		}
		break;

		case 24:
		case 32:
		{
			rledat24 *dat=(rledat24 *)rle->dat;
			_rle_wind24(dat,y);
			return(_rle_collide_point24(dat,x));
		}
		break;
	}
}

bool collide_rle_point_noclip(RLE_SPRITE *rle,int x1,int y1,int x2,int y2)
{
	int y=y2-y1,x=x2-x1;

	switch(rle->color_depth)
	{
		case 8:
		{
			rledat8 *dat=(rledat8 *)rle->dat;
			_rle_wind8(dat,y);
			return(_rle_collide_point8(dat,x));
		}
		break;

		case 15:
		{
			rledat15 *dat=(rledat15 *)rle->dat;
			_rle_wind15(dat,y);
			return(_rle_collide_point15(dat,x));
		}
		break;

		case 16:
		{
			rledat16 *dat=(rledat16 *)rle->dat;
			_rle_wind16(dat,y);
			return(_rle_collide_point16(dat,x));
		}
		break;

		case 24:
		case 32:
		{
			rledat24 *dat=(rledat24 *)rle->dat;
			_rle_wind24(dat,y);
			return(_rle_collide_point24(dat,x));
		}
		break;
	}
}



bool collide_rle_line(RLE_SPRITE *rle,int x1,int y1,int x2,int y2,int x3,int y3)
{
	int x_start,y_start,x_end,y_end,dx,dy;
	int stepsize,x_step,error,adj_up,adj_down,initial_len,final_len,len;

	if(y2<y3)
	{
		x_start=x2-x1; y_start=y2-y1;
		x_end=x3-x1; y_end=y3-y1;
	}
	else
	{
		x_start=x3-x1; y_start=y3-y1;
		x_end=x2-x1; y_end=y2-y1;
	}

	if(!clip_line(x_start,y_start,x_end,y_end,0,0,rle->w-1,rle->h-1)) return(false);

	dx=x_end-x_start;

	if(dx<0) { x_step=-1; dx=-dx; }
	else x_step=1;

	dy=y_end-y_start;

	if(dx==0)
	{
		switch(rle->color_depth)
		{
			case 8:
			{
				rledat8 *dat=(rledat8 *)rle->dat;
				_rle_wind8(dat,y_start);
				return(_rle_collide_column8(dat,x_start,dy+1));
			}
			break;

			case 15:
			{
				rledat15 *dat=(rledat15 *)rle->dat;
				_rle_wind15(dat,y_start);
				return(_rle_collide_column15(dat,x_start,dy+1));
			}
			break;

			case 16:
			{
				rledat16 *dat=(rledat16 *)rle->dat;
				_rle_wind16(dat,y_start);
				return(_rle_collide_column16(dat,x_start,dy+1));
			}
			break;

			case 24:
			case 32:
			{
				rledat24 *dat=(rledat24 *)rle->dat;
				_rle_wind24(dat,y_start);
				return(_rle_collide_column24(dat,x_start,dy+1));
			}
			break;
		}
	}
	else if(dy==0)
	{
		switch(rle->color_depth)
		{
			case 8:
			{
				rledat8 *dat=(rledat8 *)rle->dat;
				_rle_wind8(dat,y_start);
				return(_rle_collide_span8(dat,x_start,x_end));
			}
			break;

			case 15:
			{
				rledat15 *dat=(rledat15 *)rle->dat;
				_rle_wind15(dat,y_start);
				return(_rle_collide_span15(dat,x_start,x_end));
			}
			break;

			case 16:
			{
				rledat16 *dat=(rledat16 *)rle->dat;
				_rle_wind16(dat,y_start);
				return(_rle_collide_span16(dat,x_start,x_end));
			}
			break;

			case 24:
			case 32:
			{
				rledat24 *dat=(rledat24 *)rle->dat;
				_rle_wind24(dat,y_start);
				return(_rle_collide_span24(dat,x_start,x_end));
			}
			break;
		}
	}
	/*else if(dx==dy)
	{
		for(int i=0;i<=dx;i++)
		{
			putpixel(bm,x_start+i*x_step,y_start+i,col);
		}
	}*/
	else if(dx>dy)
	{
		stepsize=(dx/dy)*x_step;
		adj_up=(dx%dy)*2;
		adj_down=dy*2;
		error=adj_up/2-adj_down;
		initial_len=final_len=stepsize/2+x_step;

		if(stepsize&1) error+=dy;
		else if(adj_up==0) initial_len-=x_step;

		int x=x_start;

		switch(rle->color_depth)
		{
			case 8:
			{
				rledat8 *dat=(rledat8 *)rle->dat;
				_rle_wind8(dat,y_start);
				if(_rle_collide_span8(dat,x,x+initial_len-x_step)) return(true);
				_rle_next_line8(dat);

				x+=initial_len;

				for(int y=1;y<dy;y++)
				{
					len=stepsize; error+=adj_up;
					if(error>0) { len+=x_step; error-=adj_down; }
					if(_rle_collide_span8(dat,x,x+len-x_step)) return(true);
					_rle_next_line8(dat);
					x+=len;
				}
				if(_rle_collide_span8(dat,x,x+final_len-x_step)) return(true);
			}
			break;

			case 15:
			{
				rledat15 *dat=(rledat15 *)rle->dat;
				_rle_wind15(dat,y_start);
				if(_rle_collide_span15(dat,x,x+initial_len-x_step)) return(true);
				_rle_next_line15(dat);

				x+=initial_len;

				for(int y=1;y<dy;y++)
				{
					len=stepsize; error+=adj_up;
					if(error>0) { len+=x_step; error-=adj_down; }
					if(_rle_collide_span15(dat,x,x+len-x_step)) return(true);
					_rle_next_line15(dat);
					x+=len;
				}
				if(_rle_collide_span15(dat,x,x+final_len-x_step)) return(true);
			}
			break;

			case 16:
			{
				rledat16 *dat=(rledat16 *)rle->dat;
				_rle_wind16(dat,y_start);
				if(_rle_collide_span16(dat,x,x+initial_len-x_step)) return(true);
				_rle_next_line16(dat);

				x+=initial_len;

				for(int y=1;y<dy;y++)
				{
					len=stepsize; error+=adj_up;
					if(error>0) { len+=x_step; error-=adj_down; }
					if(_rle_collide_span16(dat,x,x+len-x_step)) return(true);
					_rle_next_line16(dat);
					x+=len;
				}
				if(_rle_collide_span16(dat,x,x+final_len-x_step)) return(true);
			}
			break;

			case 24:
			case 32:
			{
				rledat24 *dat=(rledat24 *)rle->dat;
				_rle_wind24(dat,y_start);
				if(_rle_collide_span24(dat,x,x+initial_len-x_step)) return(true);
				_rle_next_line24(dat);

				x+=initial_len;

				for(int y=1;y<dy;y++)
				{
					len=stepsize; error+=adj_up;
					if(error>0) { len+=x_step; error-=adj_down; }
					if(_rle_collide_span24(dat,x,x+len-x_step)) return(true);
					_rle_next_line24(dat);
					x+=len;
				}
				if(_rle_collide_span24(dat,x,x+final_len-x_step)) return(true);
			}
			break;
		}
	}
	else
	{
		stepsize=(dy/dx);
		adj_up=(dy%dx)*2;
		adj_down=dx*2;
		error=adj_up/2-adj_down;
		initial_len=final_len=stepsize/2+1;

		if(stepsize&1) error+=dx;
		else if(adj_up==0) initial_len-=1;

		int y=y_start;

		switch(rle->color_depth)
		{
			case 8:
			{
				rledat8 *dat=(rledat8 *)rle->dat;
				_rle_wind8(dat,y_start);
				if(_rle_collide_column8(dat,x_start,initial_len)) return(true);

				for(int x=x_start+x_step;x!=x_end;x+=x_step)
				{
					len=stepsize; error+=adj_up;
					if(error>0) { len+=1; error-=adj_down; }
					if(_rle_collide_column8(dat,x,len)) return(true);
				}
				if(_rle_collide_column8(dat,x_end,final_len)) return(true);
			}
			break;

			case 15:
			{
				rledat15 *dat=(rledat15 *)rle->dat;
				_rle_wind15(dat,y_start);
				if(_rle_collide_column15(dat,x_start,initial_len)) return(true);

				for(int x=x_start+x_step;x!=x_end;x+=x_step)
				{
					len=stepsize; error+=adj_up;
					if(error>0) { len+=1; error-=adj_down; }
					if(_rle_collide_column15(dat,x,len)) return(true);
				}
				if(_rle_collide_column15(dat,x_end,final_len)) return(true);
			}
			break;

			case 16:
			{
				rledat16 *dat=(rledat16 *)rle->dat;
				_rle_wind16(dat,y_start);
				if(_rle_collide_column16(dat,x_start,initial_len)) return(true);

				for(int x=x_start+x_step;x!=x_end;x+=x_step)
				{
					len=stepsize; error+=adj_up;
					if(error>0) { len+=1; error-=adj_down; }
					if(_rle_collide_column16(dat,x,len)) return(true);
				}
				if(_rle_collide_column16(dat,x_end,final_len)) return(true);
			}
			break;

			case 24:
			case 32:
			{
				rledat24 *dat=(rledat24 *)rle->dat;
				_rle_wind24(dat,y_start);
				if(_rle_collide_column24(dat,x_start,initial_len)) return(true);

				for(int x=x_start+x_step;x!=x_end;x+=x_step)
				{
					len=stepsize; error+=adj_up;
					if(error>0) { len+=1; error-=adj_down; }
					if(_rle_collide_column24(dat,x,len)) return(true);
				}
				if(_rle_collide_column24(dat,x_end,final_len)) return(true);
			}
			break;

		}
	}

	return(false);
}

void fill_rle_sprite(BITMAP *bm,RLE_SPRITE *rle,int x,int y,int col)
{
	switch(rle->color_depth)
	{
		case 8:
		{
			rledat8 *dat=(rledat8 *)rle->dat,val;
			for(int i=0;i<rle->h;i++)
			{
				int sx=x;
				while((val=*dat++)!=MASK_COLOR_8)
				{
					if(val<0) sx-=val;
					else { hline(bm,sx,y,sx+val-1,col); dat+=val; sx+=val; }
				}
				y++;
			}
		}
		break;

		case 15:
		{
			rledat15 *dat=(rledat15 *)rle->dat,val;
			for(int i=0;i<rle->h;i++)
			{
				int sx=x;
				while((val=*dat++)!=MASK_COLOR_15)
				{
					if(val<0) sx-=val;
					else { hline(bm,sx,y,sx+val-1,col); dat+=val; sx+=val; }
				}
				y++;
			}
		}
		break;

		case 16:
		{
			rledat16 *dat=(rledat16 *)rle->dat,val;
			for(int i=0;i<rle->h;i++)
			{
				int sx=x;
				while((unsigned short)(val=*dat++)!=MASK_COLOR_16)
				{
					if(val<0) sx-=val;
					else { hline(bm,sx,y,sx+val-1,col); dat+=val; sx+=val; }
				}
				y++;
			}
		}
		break;

		case 24:
		case 32:
		{
			rledat24 *dat=(rledat24 *)rle->dat,val;
			for(int i=0;i<rle->h;i++)
			{
				int sx=x;
				while((val=*dat++)!=MASK_COLOR_24)
				{
					if(val<0) sx-=val;
					else { hline(bm,sx,y,sx+val-1,col); dat+=val; sx+=val; }
				}
				y++;
			}
		}
		break;
	}
}
