#include <goa/list.h>



// node

node::node()
{ next=prev=0; }

node::~node() {}

void node::add_after(node *before)
{
	next=before->next;
	prev=before;
	if(next) next->prev=this;
	before->next=this;
}

void node::remove()
{
	if(next) next->prev=prev;
	if(prev) prev->next=next;
}



// list

list::list()
{
	sentinel.next=sentinel.prev=&sentinel;
}

list::~list()
{
	clear();
}


void list::add_head(node *entry)
{
	entry->add_after(&sentinel);
}

void list::add_tail(node *entry)
{
	entry->add_after(sentinel.prev);
}

void list::add_head(list *lst)
{
	if(lst->empty()) return;

	// join lists
	lst->sentinel.prev->next=sentinel.next;
	sentinel.next->prev=lst->sentinel.prev;

	// insert list at head
	sentinel.next=lst->sentinel.next;
	sentinel.next->prev=&sentinel;

	// empty other list
	lst->sentinel.next=&lst->sentinel;
	lst->sentinel.prev=&lst->sentinel;
}

void list::add_tail(list *lst)
{
	if(lst->empty()) return;

	// join lists
	sentinel.prev->next=lst->sentinel.next;
	lst->sentinel.next->prev=sentinel.prev;

	// insert list at tail
	sentinel.prev=lst->sentinel.prev;
	sentinel.prev->next=&sentinel;

	// empty other list
	lst->sentinel.next=&lst->sentinel;
	lst->sentinel.prev=&lst->sentinel;
}



node *list::find(int n)
{
	iterator i(this);
	node *e;
	while((e=i.step())&&n--);
	return(e);
}

bool list::exists(node *entry) // slow and stupid
{
	iterator i(this);
	node *e;
	while(e=i.step())
	if(e==entry) return(true);

	return(false);
}

bool list::empty()
{
	return(sentinel.next==&sentinel);
}

int list::count()
{
	int n=0;
	iterator i(this);
	node *e;
	while(e=i.step()) n++;
	return(n);
}

void list::clear()
{
	iterator i(this);
	node *e;
	while(e=i.step()) { e->remove(); delete e; }
}



// iterator

iterator::iterator()
{
}

iterator::iterator(list *lst)
{
	start(lst);
}

iterator::iterator(list *lst,node *here)
{
	start(lst,here);
}

void iterator::start(list *lst)
{
	stop=&lst->sentinel;
	next=lst->sentinel.next;
}

void iterator::start(list *lst,node *here)
{
	stop=&lst->sentinel;
	next=here?here:stop;
}

void iterator::start(node *here)
{
	next=here?here:stop;
}

void iterator::restart()
{
	next=stop->next;
}

node *iterator::step()
{
	if(next==stop) return(0);
	node *res=next;
	next=next->next;
	return(res);
}

node *iterator::peek()
{
	if(next==stop) return(0);
	return(next);
}



// reverseiterator

reverseiterator::reverseiterator()
{
}

reverseiterator::reverseiterator(list *lst)
{
	start(lst);
}

reverseiterator::reverseiterator(list *lst,node *here)
{
	start(lst,here);
}

void reverseiterator::start(list *lst)
{
	stop=&lst->sentinel;
	prev=lst->sentinel.prev;
}

void reverseiterator::start(list *lst,node *here)
{
	stop=&lst->sentinel;
	prev=here?here:stop;
}

void reverseiterator::start(node *here)
{
	prev=here?here:stop;
}

void reverseiterator::restart()
{
	prev=stop->prev;
}

node *reverseiterator::step()
{
	if(prev==stop) return(0);
	node *res=prev;
	prev=prev->prev;
	return(res);
}

node *reverseiterator::peek()
{
	if(prev==stop) return(0);
	return(prev);
}
