#include <goa/time.h>
#include <goa/types.h>


#define MAX_DT 0.1
#define CALIBRATE_MICROS 100



#if defined(_OS_UNIX_) || defined(_OS_MACOSX_)

#include <sys/time.h>

double t_prev;

void sync_delta_time()
{
	struct timeval tv;
	gettimeofday(&tv,0);
	t_prev=double(tv.tv_sec)+double(tv.tv_usec)/1000000.0;
}

double get_delta_time()
{
	struct timeval tv;
	gettimeofday(&tv,0);
	double t=double(tv.tv_sec)+double(tv.tv_usec)/1000000.0;

	double dt=t-t_prev;

	t_prev=t;

	if(dt>MAX_DT) dt=MAX_DT;

	return(dt);
}

#else
#ifdef _OS_WINDOWS_

#include <windows.h>

static double counterfreq;
static uint64 prevcount;

void sync_delta_time()
{
	if(!counterfreq)
	{
		uint64 freq;
		QueryPerformanceFrequency((PLARGE_INTEGER)&freq);

		counterfreq=double(freq);
	}

	QueryPerformanceCounter((PLARGE_INTEGER)&prevcount);
}

double get_delta_time()
{
	uint64 count;

	QueryPerformanceCounter((PLARGE_INTEGER)&count);

	uint64 delta=count-prevcount;

	prevcount=count;

	double dt=double(delta)/counterfreq;

	if(dt>MAX_DT) dt=MAX_DT;

	return(dt);
}


#else

#include <allegro.h>

/*inline uint32 get_clock_32()
{
	uint32 ticks_lower,ticks_upper;

	asm volatile ("rdtsc":"=a"(ticks_lower),"=d"(ticks_upper));

	return(ticks_lower);
}*/

inline uint64 get_clock_64()
{
	uint32 ticks_lower,ticks_upper;

	asm volatile ("rdtsc":"=a"(ticks_lower),"=d"(ticks_upper));

	return((uint64(ticks_upper)<<32)+uint64(ticks_lower));
}



static double calibrateclock()
{
	rest(5); // compensate for Win9x timer graininess
	uint64 start=get_clock_64();
	rest(CALIBRATE_MICROS);
	uint64 end=get_clock_64();

	uint64 diff=end-start;

	double freq=double(diff)*1000.0/CALIBRATE_MICROS;

	return(freq);
}

static double clockfreq=0;
static uint64 prevclock;

void sync_delta_time()
{
	if(!clockfreq) clockfreq=calibrateclock();

	prevclock=get_clock_64(); // hmm...
}

double get_delta_time()
{
	uint64 clock=get_clock_64();
	uint64 delta=clock-prevclock;

	prevclock=clock;

	double dt=double(delta)/clockfreq;

	if(dt>MAX_DT) dt=MAX_DT;

	return(dt);
}

#endif
#endif



#define BUFSIZE 32

double calc_fps(double dt)
{
	static double fpsbuf[BUFSIZE]={0};
	static int bufptr=0;
	double framelen=0;

	fpsbuf[bufptr]=dt;
	bufptr=(bufptr+1)%BUFSIZE;

	for(int i=0;i<BUFSIZE;i++)
	framelen+=fpsbuf[i];
	framelen/=BUFSIZE;

	return(1/framelen);
}
