#ifndef _GOA_COMPLEX_TEMPLATE_H_
#define _GOA_COMPLEX_TEMPLATE_H_

#include <math.h>

#define complex complex_t<number>
#define templ template <class number>

template <class number> class complex_t
{
	public:
	complex_t();
	complex_t(complex const &other);
	complex_t(number real);
	complex_t(number real,number imag);

	complex const &operator=(const complex &other);
	complex const &operator=(const number other);

	complex const operator+(complex const &other) const;
	complex const operator-(complex const &other) const;
	complex const operator-() const;
	complex const operator*(complex const &other) const;
	complex const operator/(complex const &other) const;

	complex const &operator+=(complex const &other);
	complex const &operator+=(number const other);
	complex const &operator-=(complex const &other);
	complex const &operator-=(number const other);
	complex const &operator*=(complex const &other);
	complex const &operator*=(number const other);
	complex const &operator/=(complex const &other);
	complex const &operator/=(number const other);

	bool const operator==(complex const &other) const;

	number abs() const;
	number abs2() const;
	complex const conj() const;
	complex const conjugated() const;
	complex const &conjugate();
	complex const normalized() const;
	complex const &normalize();
	complex const inverse() const;
	complex const &invert();

	complex const exp() const;
//	complex const sin() const;
//	complex const cos() const;
//	complex const tan() const;
//	complex const sinh() const;
//	complex const cosh() const;
//	complex const tanh() const;
//	complex const ln() const;
//	complex const sqrt() const;
//	complex const pow(complex const &other) const;

	number re,im;
};



// constructors

templ complex::complex_t() { re=im=0; }
templ complex::complex_t(complex const &other) { *this=other; }
templ complex::complex_t(number real) { re=real; im=0; }
templ complex::complex_t(number real,number imag) { re=real; im=imag; }



// operators

templ complex const &complex::operator=(complex const &other)
{ re=other.re; im=other.im; return(*this); }
templ complex const &complex::operator=(number const other)
{ re=other; im=0; return(*this); }



templ complex const complex::operator+(complex const &other) const
{ return(complex(re+other.re,im+other.im)); }
templ complex const operator+(complex const &left,number const right)
{ return(complex(left.re+right,left.im)); }
templ complex const operator+(number const left,complex const &right)
{ return(complex(left+right.re,right.im)); }

templ complex const complex::operator-(complex const &other) const
{ return(complex(re-other.re,im-other.im)); }
templ complex const operator-(complex const &left,number const right)
{ return(complex(left.re-right,left.im)); }
templ complex const operator-(number const left,complex const &right)
{ return(complex(left-right.re,-right.im)); }
templ complex const complex::operator-() const
{ return(complex(-re,-im)); }

templ complex const complex::operator*(complex const &other) const
{ return(complex(re*other.re-im*other.im,re*other.im+im*other.re)); }
templ complex const operator*(complex const &left,number const right)
{ return(complex(left.re*right,left.im*right)); }
templ complex const operator*(number const left,complex const &right)
{ return(complex(left*right.re,left*right.im)); }

templ complex const complex::operator/(complex const &other) const
{ return(complex(re*other.re+im*other.im,-re*other.im+im*other.re)/other.abs2()); }
templ complex const operator/(complex const &left,number const right)
{ return(complex(left.re/right,left.im/right)); }
templ complex const operator/(number const left,complex const &right)
{ return(complex(left*right.re,-left*right.im)/right.abs2()); }



templ complex const &complex::operator+=(complex const &other)
{ re+=other.re; im+=other.im; return(*this); }
templ complex const &complex::operator+=(number const other)
{ re+=other; return(*this); }

templ complex const &complex::operator-=(complex const &other)
{ re-=other.re; im-=other.im; return(*this); }
templ complex const &complex::operator-=(number const other)
{ re-=other; return(*this); }

templ complex const &complex::operator*=(complex const &other)
{ return(*this=*this*other); }
templ complex const &complex::operator*=(number const other)
{ re*=other; im*=other; return(*this); }

templ complex const &complex::operator/=(complex const &other)
{ return(*this=*this/other); }
templ complex const &complex::operator/=(number const other)
{ re/=other; im/=other; return(*this); }



templ bool const complex::operator==(complex const &other) const
{ return(re==other.re&&im==other.im); }
templ complex const operator==(complex const &left,number const right)
{ return(left.re==right&&left.im==0); }
templ complex const operator==(number const left,complex const &right)
{ return(left==right.re&&0==right.im); }



// basic functions

templ number complex::abs() const
{ return(sqrt(re*re+im*im)); }

templ number complex::abs2() const
{ return(re*re+im*im); }

templ complex const complex::conj() const
{ return(complex(re,-im)); }

templ complex const complex::conjugated() const
{ return(complex(re,-im)); }

templ complex const &complex::conjugate()
{ im=-im; return(*this); }

templ complex const complex::normalized() const
{
	number inv=1/abs();
	return(complex(re*inv,im*inv));
}

templ complex const &complex::normalize()
{
	number inv=1/abs();
	re*=inv; im*=inv;
	return(*this);
}

templ complex const complex::inverse() const
{ return(conj()/abs2()); }

templ complex const &complex::invert()
{ conjugate(); *this/=abs2(); return(*this); }



// mathematical functions

templ complex const complex::exp() const
{ return(complex(cos(im),sin(im))*::exp(re)); }



#undef complex
#undef templ



#endif

