#ifndef _GOA_GUI_H_
#define _GOA_GUI_H_

#include <allegro.h>
#include <stdarg.h>

#include "list.h"

/*

DO THIS:

! Fix and figure out dynamic adding and removing of objects + re-layouting.
! Partial redraws, different redraw method (no backing bitmap etc)

* "Registry"

* Gadgets:
  - Text
  - Image
  - Text+image combo?
  - String
  - Virtual
    - Virtualcontainer
    - Listview

* Windowing system gadget
  - Better message passing needed for window controls etc.
  - Window gadget, dragbar gadget, close gadget... etc.
  - If this is a group object, there needs to be a system to rearrange objects
    in a group

* Timer system?

* MUI like constructors?

*/

#define GUIRES_EXIT -1
#define GUIRES_ERROR -2

#define GUIFLAG_ACTIVE 0x0001
#define GUIFLAG_REDRAW 0x0002
#define GUIFLAG_FOCUSED 0x0004
#define GUIFLAG_MOUSEOVER 0x0008
#define GUIFLAG_PRESSED 0x0010
#define GUIFLAG_CLICKABLE 0x0020
#define GUIFLAG_TOGGLE 0x0040
#define GUIFLAG_NOMOUSE 0x0080
#define GUIFLAG_USER 0x0100

#define GUIPLACE_CENTER 0x00
#define GUIPLACE_LEFT 0x01
#define GUIPLACE_RIGHT 0x02
#define GUIPLACE_LEFT_GOLDEN_MEAN 0x03
#define GUIPLACE_RIGHT_GOLDEN_MEAN 0x04
#define GUIPLACE_CENTER_CONTENTS 0x05 // skinframe title placing only

#define GUIPLACE_HORIZ_NONE 0x0f
#define GUIPLACE_HORIZ_MASK 0x0f

#define GUIPLACE_MIDDLE 0x00
#define GUIPLACE_TOP 0x10
#define GUIPLACE_BOTTOM 0x20
#define GUIPLACE_TOP_GOLDEN_MEAN 0x30
#define GUIPLACE_BOTTOM_GOLDEN_MEAN 0x40
#define GUIPLACE_MIDDLE_CONTENTS 0x50 // skinframe title placing only

#define GUIPLACE_VERT_NONE 0xf0
#define GUIPLACE_VERT_MASK 0xf0

#define GUIPLACE_NONE (GUIPLACE_HORIZ_NONE|GUIPLACE_VERT_NONE)

#define GUINAVIGATE_FORWARD 0x01
#define GUINAVIGATE_BACKWARD 0x02

#define GUI_NO_MAX_SIZE 0x7fffffff

#define GUITRIGGER_LEFT 1
#define GUITRIGGER_RIGHT 2
#define GUITRIGGER_KEY 3



// basic gadget

class g_frame;

class g_gadget:public node
{
	public:
	g_gadget();
	~g_gadget();

	// basic functions
	virtual void become_child(g_gadget *newparent);
	virtual void calc_sizes();
	virtual void resize(int w,int h);
	virtual void draw(BITMAP *bm,int x,int y,int w,int h);

	// input functions
	virtual bool key(int c,int k);
	virtual bool mouse_click(int x,int y,int b,int ch);
	virtual bool mouse_release(int x,int y,int b,int ch);
	virtual void mouse_move(int x,int y);
	virtual void mouse_in(int x,int y);
	virtual void mouse_out(int x,int y);
	virtual void trigger(int type);

	// focus functions
	virtual bool want_focus();
	virtual bool release_focus();
	virtual void get_focus();
	virtual void lose_focus();
	virtual void navigate_into(int type);

	// requests
	virtual void redraw();
//	virtual void redraw(int x,int y,int w,int h);
	virtual void event(int event);
//	virtual bool request_focus();
//	virtual g_gadget find_root();

	// flag helpers
	bool get_flag(int val);
	void set_flag(int val);
	void clear_flag(int val);
	void toggle_flag(int val);

	// event handler
	int run(int x0=0,int y0=0);
	bool startup(int x0=0,int y0=0);
	void shutdown();
	int handle_events();
	int wait_events();

	// helper funcs
	void calc_placing(int x0,int y0,int w0,int h0);

	// options
	int place,flags,fixwidth,fixheight,ev;
	g_frame *frame;

	// for use by the lauout engine
	int minwidth,minheight,maxwidth,maxheight;
	int x,y,w,h;

	// internal
	g_gadget *parent;
	bool pressing;

	// extra info for the event server
	class extinfo
	{
		public:
		extinfo();
		extinfo(int w,int h);
		~extinfo();

		BITMAP *back,*tmp;
		int event;
		int old_mx,old_my,old_mb;
	} *ext;

/*

	// void become_child(m_menuet *newparent);
	void layout();
	void draw(BITMAP *bm,int x,int y,int w,int h);

	bool key(int c,int k);
	bool mouse_click(int x,int y,int b,int ch);
	bool mouse_release(int x,int y,int b,int ch);
	void mouse_move(int x,int y);
	void mouse_in(int x,int y);
	void mouse_out(int x,int y);

	bool want_focus();
	// bool release_focus();
	// void get_focus();
	// void lose_focus();

	// void redraw();
	// void trigger(int event);
*/
};



// groups

#define insert_va(first) { insert_single(first); va_list v; g_gadget *g; va_start(v,first); while(g=va_arg(v,g_gadget *)) insert_single(g); va_end(v); }

class g_group:public g_gadget
{
	public:
	g_group();
	~g_group();

	void calc_sizes();
	void resize(int w,int h);
	void draw(BITMAP *bm,int x,int y,int w,int h);

	bool key(int c,int k);
	bool mouse_click(int x,int y,int b,int ch);
	bool mouse_release(int x,int y,int b,int ch);
	void mouse_move(int x,int y);

	bool want_focus();
	bool release_focus();
	void get_focus();
	void lose_focus();
	void navigate_into(int type);

	void insert_single(g_gadget *gadget);
	void insert(g_gadget **gadgets);
	void insert(g_gadget *first,...);

	g_gadget *find_gadget(int x,int y);

	bool set_focus(g_gadget *newfocus);
	bool focus_next(bool loop);
	bool focus_prev(bool loop);

	list children;
	g_gadget *focus,*mouseover;
};

class g_gridgroup:public g_group
{
	public:
	g_gridgroup(int size,bool fixedrows=false,bool vertical=false);
	~g_gridgroup();

	void calc_sizes();
	void resize(int w,int h);

	virtual void distribute_space(int w,int h);

	int hspace,vspace;

	bool fixrows,vert;
	int num,cols,rows;
	int *minwidths,*minheights;
	int *maxwidths,*maxheights;
	int *usewidths,*useheights;
};

class g_pagegroup:public g_group
{
	public:
};



// virtual gadgets

// moving the viewport should generate v_mouse_move()s

class g_virtual:public g_gadget
{
	public:
	g_virtual();

	void calc_sizes();
	void resize(int w,int h);
	void draw(BITMAP *bm,int x,int y,int w,int h);

	bool key(int c,int k);
	bool mouse_click(int x,int y,int b,int ch);
	bool mouse_release(int x,int y,int b,int ch);
	void mouse_move(int x,int y);

	virtual void v_calc_sizes();
	virtual void v_resize(int w,int h);
	virtual void v_draw(BITMAP *bm,int x,int y,int w,int h,int x_offs,int y_offs,int w_vis,int h_vis);
	virtual bool v_mouse_click(int x,int y,int b,int ch);
	virtual bool v_mouse_release(int x,int y,int b,int ch);
	virtual void v_mouse_move(int x,int y);

	int stepsize;

	int x0,y0;
	int v_minwidth,v_minheight;
	int v_maxwidth,v_maxheight;

	int v_view_width,v_view_height;
	int v_width,v_height;
};

class g_container:public g_virtual
{
	public:
	g_container();
	g_container(g_gadget *content);
	~g_container();

	bool want_focus();
	bool release_focus();
	void get_focus();
	void lose_focus();
	void navigate_into(int type);

	void v_calc_sizes();
	void v_resize(int w,int h);
	void v_draw(BITMAP *bm,int x,int y,int w,int h,int x_offs,int y_offs,int w_vis,int h_vis);
	bool v_mouse_click(int x,int y,int b,int ch);
	bool v_mouse_release(int x,int y,int b,int ch);
	void v_mouse_move(int x,int y);

	void set_content(g_gadget *content);

	g_gadget *g;
};




// interface components

class g_text:public g_gadget
{
	public:
	g_text(const char *text=0);
	~g_text();

	void calc_sizes();
	void draw(BITMAP *bm,int x,int y,int w,int h);

	void set_text(const char *text);

	FONT *f;
	int ncol,fcol,pcol,icol;

	int numlines,maxline;
	char **lines;
};

class g_button:public g_text
{
	public:
	g_button(const char *txt,int event=0);
};

/*
class g_image:public g_gadget
{
};*/

//class g_listview:public g_virtual
//{
//};

class g_scrollbar:public g_gadget
{
};

class g_rectangle:public g_gadget
{
	public:
	g_rectangle(int width=0,int height=0);

	void calc_sizes();

	int hspace,vspace;
};

class g_spacer:public g_rectangle
{
	public:
	g_spacer(int width=0,int height=0);
};



// frames

class g_frame
{
	public:
	g_frame();

	virtual void draw_normal(BITMAP *bm,int x1,int y1,int x2,int y2);
	virtual void draw_focused(BITMAP *bm,int x1,int y1,int x2,int y2);
	virtual void draw_pressed(BITMAP *bm,int x1,int y1,int x2,int y2);
	virtual void draw_inactive(BITMAP *bm,int x1,int y1,int x2,int y2);

	int left,right,top,bottom;
};

class g_noframe:public g_frame
{
	public:
	g_noframe();

	virtual void draw_normal(BITMAP *bm,int x1,int y1,int x2,int y2);
	virtual void draw_focused(BITMAP *bm,int x1,int y1,int x2,int y2);
	virtual void draw_pressed(BITMAP *bm,int x1,int y1,int x2,int y2);
	virtual void draw_inactive(BITMAP *bm,int x1,int y1,int x2,int y2);

	int back;
};

class g_happyframe:public g_frame
{
	public:
	g_happyframe(bool rounded=false);

	void draw(BITMAP *bm,int x1,int y1,int x2,int y2,int ol,int il,int b,int ir,int or_);
	void draw_square(BITMAP *bm,int x1,int y1,int x2,int y2,int ol,int il,int b,int ir,int or_);
	void draw_round(BITMAP *bm,int x1,int y1,int x2,int y2,int ol,int il,int b,int ir,int or_);

	int back,shine,shadow,halfshine,halfshadow,highlight;
	bool round;
};

class g_happyframe_button:public g_happyframe
{
	public:
	g_happyframe_button(bool rounded=false);

	void draw_normal(BITMAP *bm,int x1,int y1,int x2,int y2);
	void draw_focused(BITMAP *bm,int x1,int y1,int x2,int y2);
	void draw_pressed(BITMAP *bm,int x1,int y1,int x2,int y2);
	void draw_inactive(BITMAP *bm,int x1,int y1,int x2,int y2);
};

class g_happyframe_group:public g_happyframe
{
	public:
	g_happyframe_group(bool rounded=false);

	void draw_normal(BITMAP *bm,int x1,int y1,int x2,int y2);
	void draw_focused(BITMAP *bm,int x1,int y1,int x2,int y2);
	void draw_pressed(BITMAP *bm,int x1,int y1,int x2,int y2);
	void draw_inactive(BITMAP *bm,int x1,int y1,int x2,int y2);
};









/*class g_look
{
	public:
	g_look();
	~g_look();

	void get_look(g_gadget *com,g_looktype looktype);

	gframe *nullframe;
	FONT *deffont;

	gframe *frames[GLOOK_NUM][GFRAME_NUM];
	FONT *fonts[GLOOK_NUM];
};

class happylook:public glook
{
	public:
	happylook();

	int back,shine,shadow,halfshine,halfshadow,highlight,lowlight,text;
};


class happyframe_square:public gframe
{
	public:
	happyframe_square(int outerleft,int innerleft,int back,int innerright,int outerright);

	void draw(BITMAP *bm,int x1,int y1,int x2,int y2);

	int ol,il,b,ir,or;
};

class happyframe_round:public gframe
{
	public:
	happyframe_round(int outerleft,int innerleft,int back,int innerright,int outerright);

	void draw(BITMAP *bm,int x1,int y1,int x2,int y2);

	int ol,il,b,ir,or;
};
*/

#endif
