#ifndef _GOA_RLE_INLINE_H_
#define _GOA_RLE_INLINE_H_

typedef char rledat8;
typedef short rledat15;
typedef short rledat16;
typedef long rledat24;


static inline void _rle_next_line8(rledat8 *&dat)
{ while(*dat++!=MASK_COLOR_8); }

static inline void _rle_next_line15(rledat15 *&dat)
{ while(*dat++!=MASK_COLOR_15); }

static inline void _rle_next_line16(rledat16 *&dat)
{ while((unsigned short)(*dat++)!=MASK_COLOR_16); }

static inline void _rle_next_line24(rledat24 *&dat)
{ while(*dat++!=MASK_COLOR_24); }



static inline void _rle_wind8(rledat8 *&dat,int lines)
{ while(lines) if(*dat++==MASK_COLOR_8) lines--; }

static inline void _rle_wind15(rledat15 *&dat,int lines)
{ while(lines) if(*dat++==MASK_COLOR_15) lines--; }

static inline void _rle_wind16(rledat16 *&dat,int lines)
{ while(lines) if((unsigned short)(*dat++)==MASK_COLOR_16) lines--; }

static inline void _rle_wind24(rledat24 *&dat,int lines)
{ while(lines) if(*dat++==MASK_COLOR_24) lines--; }



static inline void _rle_find_solid8(rledat8 *&dat,int &x)
{ while(*dat!=MASK_COLOR_8&&*dat<0) { x-=*dat; dat++; } }

static inline void _rle_find_solid15(rledat15 *&dat,int &x)
{ while(*dat!=MASK_COLOR_15&&*dat<0) { x-=*dat; dat++; } }

static inline void _rle_find_solid16(rledat16 *&dat,int &x)
{ while((unsigned short)(*dat)!=MASK_COLOR_16&&*dat<0) { x-=*dat; dat++; } }

static inline void _rle_find_solid24(rledat24 *&dat,int &x)
{ while(*dat!=MASK_COLOR_24&&*dat<0) { x-=*dat; dat++; } }



static inline void _rle_find_solid_rev8(rledat8 *&dat,int &x)
{ while(*dat!=MASK_COLOR_8&&*dat<0) { x+=*dat; dat++; } }

static inline void _rle_find_solid_rev15(rledat15 *&dat,int &x)
{ while(*dat!=MASK_COLOR_15&&*dat<0) { x+=*dat; dat++; } }

static inline void _rle_find_solid_rev16(rledat16 *&dat,int &x)
{ while((unsigned short)(*dat)!=MASK_COLOR_16&&*dat<0) { x+=*dat; dat++; } }

static inline void _rle_find_solid_rev24(rledat24 *&dat,int &x)
{ while(*dat!=MASK_COLOR_24&&*dat<0) { x+=*dat; dat++; } }



static inline bool _rle_collide_point8(rledat8 *&dat,int x)
{
	rledat8 val;
	while((val=*dat)!=MASK_COLOR_8)
	{
		if(val>0) { dat+=val; x-=val; if(x<0) return(true); }
		else { x+=val; if(x<0) return(false); }
		dat++;
	}
	return(false); // please don't happen
}

static inline bool _rle_collide_point15(rledat15 *&dat,int x)
{
	rledat15 val;
	while((val=*dat)!=MASK_COLOR_15)
	{
		if(val>0) { dat+=val; x-=val; if(x<0) return(true); }
		else { x+=val; if(x<0) return(false); }
		dat++;
	}
	return(false); // please don't happen
}

static inline bool _rle_collide_point16(rledat16 *&dat,int x)
{
	rledat15 val;
	while((unsigned short)(val=*dat)!=MASK_COLOR_16)
	{
		if(val>0) { dat+=val; x-=val; if(x<0) return(true); }
		else { x+=val; if(x<0) return(false); }
		dat++;
	}
	return(false); // please don't happen
}

static inline bool _rle_collide_point24(rledat24 *&dat,int x)
{
	rledat24 val;
	while((val=*dat)!=MASK_COLOR_24)
	{
		if(val>0) { dat+=val; x-=val; if(x<0) return(true); }
		else { x+=val; if(x<0) return(false); }
		dat++;
	}
	return(false); // please don't happen
}




static inline bool _rle_collide_column8(rledat8 *&dat,const int x,int y)
{
	while(y--)
	{
		if(_rle_collide_point8(dat,x)) return(true);
		_rle_next_line8(dat);
	}
	return(false);
}

static inline bool _rle_collide_column15(rledat15 *&dat,const int x,int y)
{
	while(y--)
	{
		if(_rle_collide_point15(dat,x)) return(true);
		_rle_next_line15(dat);
	}
	return(false);
}

static inline bool _rle_collide_column16(rledat16 *&dat,const int x,int y)
{
	while(y--)
	{
		if(_rle_collide_point16(dat,x)) return(true);
		_rle_next_line16(dat);
	}
	return(false);
}

static inline bool _rle_collide_column24(rledat24 *&dat,const int x,int y)
{
	while(y--)
	{
		if(_rle_collide_point24(dat,x)) return(true);
		_rle_next_line24(dat);
	}
	return(false);
}



static inline bool _rle_collide_span8(rledat8 *&dat,int x1,int x2)
{
	if(x1>x2) { int tmp=x1; x1=x2; x2=tmp; }
	int x=0;
	rledat8 val;
	while((val=*dat)!=MASK_COLOR_8)
	{
		if(val>0) { if(x<=x2&&(x+=val)>x1) return(true); dat+=val; }
		else { x-=val; if(x>x2) return(false); }
		dat++;
	}
}

static inline bool _rle_collide_span15(rledat15 *&dat,int x1,int x2)
{
	if(x1>x2) { int tmp=x1; x1=x2; x2=tmp; }
	int x=0;
	rledat15 val;
	while((val=*dat)!=MASK_COLOR_15)
	{
		if(val>0) { if(x<=x2&&(x+=val)>x1) return(true); dat+=val; }
		else { x-=val; if(x>x2) return(false); }
		dat++;
	}
}

static inline bool _rle_collide_span16(rledat16 *&dat,int x1,int x2)
{
	if(x1>x2) { int tmp=x1; x1=x2; x2=tmp; }
	int x=0;
	rledat16 val;
	while((unsigned short)(val=*dat)!=MASK_COLOR_16)
	{
		if(val>0) { if(x<=x2&&(x+=val)>x1) return(true); dat+=val; }
		else { x-=val; if(x>x2) return(false); }
		dat++;
	}
}

static inline bool _rle_collide_span24(rledat24 *&dat,int x1,int x2)
{
	if(x1>x2) { int tmp=x1; x1=x2; x2=tmp; }
	int x=0;
	rledat24 val;
	while((val=*dat)!=MASK_COLOR_24)
	{
		if(val>0) { if(x<=x2&&(x+=val)>x1) return(true); dat+=val; }
		else { x-=val; if(x>x2) return(false); }
		dat++;
	}
}

#endif
