#include <stdio.h>
#include <allegro.h>

#include <goa/misc.h>

int column=0;

void print_byte(int byte)
{
	if(column==0) printf("\n\t");
	printf("0x%02x,",byte&0xff);
	column=(column+1)&15;
}

void convert(char *filename)
{
	PALETTE pal,optpal;
	BITMAP *bm=load_bitmap(filename,pal);
	int ncol,coltable[256];

	if(!bm)
	{
		fprintf(stderr,"Couldn't load image \"%s\". Sorry.\n",filename);
		exit(1);
	}

	ncol=0;
	for(int i=1;i<256;i++)
	{
		int j;
		for(j=0;j<ncol;j++)
		if(	pal[i].r==optpal[j].r&&
			pal[i].g==optpal[j].g&&
			pal[i].b==optpal[j].b) break;

		if(j==ncol) // not found
		{
			ncol++;
			optpal[j].r=pal[i].r;
			optpal[j].g=pal[i].g;
			optpal[j].b=pal[i].b;
		}
		coltable[i]=j;
	}
	coltable[0]=-1;

	if(ncol>127)
	{
		fprintf(stderr,"Too many colours in image \"%s\". Sorry.\n",filename);
		exit(1);
	}

	printf("unsigned char pic[]=\n{");

	column=0;

	print_byte((bm->w>>8)&0xff);
	print_byte(bm->w&0xff);
	print_byte((bm->h>>8)&0xff);
	print_byte(bm->h&0xff);
	print_byte(ncol);

	for(int i=0;i<ncol;i++)
	{
		print_byte(optpal[i].r<<2);
		print_byte(optpal[i].g<<2);
		print_byte(optpal[i].b<<2);
	}

	int w=bm->w,h=bm->h;

	for(int y=0;y<h;y++)
	{
		int x=0;

		while(x<w)
		{
			int val=getpixel(bm,x,y);
			int span=1;

			while(getpixel(bm,x+span,y)==val) span++;

			x+=span;

			if(span==1)
			{
				if(val==0) print_byte(-1);
				print_byte(coltable[val]);
			}
			else
			{
				while(span>127)
				{
					print_byte(-127);
					print_byte(coltable[val]);
					span-=127;
				}
				print_byte(-span);
				print_byte(coltable[val]);
			}
		}
	}

	printf("\n};\n");
}

int main(int argc,char **argv)
{
	if(argc==1)
	{
		fprintf(stderr,"Usage: %s file1.(bmp|tga|pcx) [file2 ...]",argv[0]);
	}

	if(allegro_init()) die("allegro_init()");
	set_color_depth(8);

	for(int i=1;i<argc;i++)
	{
		if(i!=1) printf("\n");
		convert(argv[i]);
	}
}
