#include <time.h>
#include <goa/allegro.h>
#include <goa/input.h>
#include <goa/menu.h>
#include <goa/types.h>

#include "main.h"
#include "game.h"
#include "swearconfig.h"
#include "playfield.h"
#include "levels.h"



void mainmenu();
void mergescores();
void make_fade(int first,int num,int r1,int g1,int b1,int r2,int g2,int b2);
void make_cmap(COLOR_MAP *cmap);



m_menulook look_ni,look_hi,look_small_ni;
BITMAP *bm;

FONT *victor_10,*victor_12;


int main()
{
	srand(time(NULL));
	if(allegro_init()) die("allegro_init() failed");
	if(install_mouse()==-1) die("install_mouse() failed");
	install_timer();
	install_keyboard();
	if(load_joystick_data(0)) // failure
	install_joystick(JOY_TYPE_AUTODETECT);

	install_fancy_sighandler(crash_guru);

//	install_sound(DIGI_AUTODETECT,MIDI_AUTODETECT,0); // should work even if this fails?

	init_descriptions();

	c=new swearconfig();

	int mode=c->windowed?GFX_AUTODETECT_WINDOWED:GFX_AUTODETECT_FULLSCREEN;

	set_color_depth(8);
	if(set_gfx_mode(mode,640,480,0,0)<0)
	{
		if(set_gfx_mode(GFX_AUTODETECT,640,480,0,0)<0) die("Screen mode not supported");
	}

	set_window_title(NAME " " VERSION);

	bm=create_bitmap(SCREEN_W,SCREEN_H);
	if(!bm) die("Couldn't create bitmap");

	DATAFILE *data=load_datafile("swear.dat");
	victor_10=(FONT *)data[0].dat;
	victor_12=(FONT *)data[1].dat;

	PALETTE pal=
	{
		{0,3,0},
		{0,63,0},
		{47,63,47},
		{0,31,0},
		{63,0,0},
	};
	set_palette(pal);
//	make_fade(0,64,    0, 0, 0,   0,  0,  0);
	make_fade(31,33,   0, 0, 0, 255,255,255);
	make_fade(64,64,   0,31, 0,   0,127,  0);
	make_fade(128,64, 63,31,31, 255,127,127);
	make_fade(192,64, 63, 0, 0, 255, 63, 63);

	COLOR_MAP cmap;
//	get_palette(pal);
//	set_fast_screen_blender(0,0,0);
//	for(int i=0;i<64;i++) pal[i].r=pal[i].g=pal[i].b=0;
//	create_blender_table(&cmap,pal,0);
	make_cmap(&cmap);
	color_map=&cmap;

/*	PALETTE pal2=
	{
		{63,63,63},
		{0,0,0},
		{63,0,0},
		{31,31,31},
	};
	set_palette(pal2);
	make_fade(31,33,255,255,255,0,0,0);
	make_fade(64,64,223,223,223,0,0,0);
	make_fade(128,64,255,223,223,255,0,0);
	make_fade(192,64,223,223,255,0,0,255);*/

	gui_fg_color=1;
	gui_bg_color=0;
	font=victor_10;

	text_mode(-1);

	mainmenu();

	unload_datafile(data);

	destroy_bitmap(bm);

	delete c;

	return(0);
}

END_OF_MAIN();

int n=0;

void mainmenu()
{
	while(levels[n]) n++;

	bool running=true;

	m_look.f=victor_12;
	m_look.ncol=1;
	m_look.fcol=2;
	m_look.pcol=3;
	m_look.icol=3;

	look_ni.f=victor_12;
	look_ni.ncol=1;
	look_ni.fcol=2;
	look_ni.pcol=1;
	look_ni.icol=1;

	look_hi.f=victor_12;
	look_hi.ncol=2;
	look_hi.fcol=1;
	look_hi.pcol=2;
	look_hi.icol=2;

	look_small_ni.f=victor_10;
	look_small_ni.ncol=1;
	look_small_ni.fcol=2;
	look_small_ni.pcol=1;
	look_small_ni.icol=1;

	m_text *cont;
	m_multi *level,*type;
	m_menu *m=new m_menu(
		new m_label(NAME " " VERSION,&look_ni),
		new m_label("by circleSoft",&look_small_ni),
		new g_spacer(0,8),
		//new m_string("Test: ",100,MAX_NAMELEN,0,0),
		cont=new m_text("Continue game",1,0),
		new m_text("New game",2,0),
		level=new m_multi("Surface: ",0,n+1),
		type=new m_multi("Game type: ",0,"Traditional","Hunt","Blind",0),
		new g_spacer(0,4),
		new m_text("Highscores",3,0),
		new m_text("Options",4,0),
		new m_text("Quit",5,0),
		0
	);
	m->place=GUIPLACE_CENTER|GUIPLACE_MIDDLE;
	m->set_flag(GUIFLAG_NOMOUSE);

	for(int i=0;i<n;i++)
	{
		level->insert(levels[i]->name);
		levels[i]->loadhighscores();
	}
	level->insert("Random");

	game *games[]=
	{
		new simplegame(),
		new huntgame(),
		new blindgame(),
		0
	};

	// ugly startup screen handling
/*	g=new startup();
	g->setup();
	g->play();
	delete g;*/

	g=games[0];

	while(running)
	{
		bool newgame=false,play=false;

		if(g->playing) cont->set_flag(GUIFLAG_ACTIVE);
		else cont->clear_flag(GUIFLAG_ACTIVE);

//swear_antialias_line(screen,10,10,200,100,100);

		while(key[KEY_ESC]); // YUK! FIX THE DAMN KEYBOARD INPUT CODE ALREADY!

		switch(m->run())
		{
			case 1:
				play=true;
			break;

			case 2:
				play=true;
				newgame=true;
			break;

			case 3:
				if(level->i<n)
				levels[level->i]->showhighscores(type->i,-1);
			break;

			case 4:
				c->run_menu();
			break;

			case 5:
				running=false;
			break;

			case -1:
				if(g->playing) play=true;
				else running=false;
			break;
		}

		if(play) do
		{
			if(newgame)
			{
				g=games[type->i];

				if(level->i<n) g->p=levels[level->i];
				else g->p=levels[rand()%n];

				if(!g->p->prepared) g->p->prepare(); // avoid funny bug

				g->setup();
			}
			else if(!g->p->prepared) g->p->prepare();

			g->play();
			newgame=true;
		} while(!g->playing&&c->autorestart);
	}

	for(int i=0;games[i];i++)
	delete games[i];

	delete m;

	for(int i=0;i<n;i++)
	delete levels[i];
}



void mergescores()
{
	char file[MAX_STRLEN]="";

	BITMAP *back=create_bitmap(SCREEN_W,SCREEN_H);
	if(back) blit(screen,back,0,0,0,0,SCREEN_W,SCREEN_H);

	if(file_select("Select a score file",file,"cfg"))
	{
		for(int i=0;i<n;i++)
		levels[i]->mergehighscores(file);
	}

	if(back) blit(back,screen,0,0,0,0,SCREEN_W,SCREEN_H);

	text_mode(-1);
}

void unprepare_all()
{
	for(int i=0;i<n;i++)
	{
		levels[i]->unprepare();
	}
}



void make_fade(int first,int num,int r1,int g1,int b1,int r2,int g2,int b2)
{
	RGB col;

	for(int i=0;i<num;i++)
	{
		col.r=(r1*(num-1-i)+r2*i)/(num-1)/4;
		col.g=(g1*(num-1-i)+g2*i)/(num-1)/4;
		col.b=(b1*(num-1-i)+b2*i)/(num-1)/4;

		set_color(first+i,&col);
	}
}

void make_cmap(COLOR_MAP *cmap)
{
	PALETTE pal;

	get_palette(pal);

	for(int i=0;i<256;i++)
	for(int j=0;j<256;j++)
	{
		int gi=i>>6,gj=j>>6;

		if(gi==gj)
		switch(gi)
		{
			case 0:
				cmap->data[i][j]=MAX(i,j);
			break;

			case 1: // wire
			{
				int v1=pal[i].g;
				int v2=pal[j].g;
				int v=63-(63-v1)*(63-v2)/63;
				int n=0,dist=64;
				for(int c=64;c<128;c++)
				if(ABS(pal[c].g-v)<dist)
				{
					n=c;
					dist=ABS(pal[c].g-v);
				}

				cmap->data[i][j]=n;
			}
			break;

			case 2: // worm
			{
				int v1=pal[i].r+pal[i].g+pal[i].b;
				int v2=pal[j].r+pal[j].g+pal[j].b;
				cmap->data[i][j]=v1>v2?i:j;
//				cmap->data[i][j]=(i+j)/2;
			}
			break;

			case 3: // apple
			{
				int r1=pal[i].r,g1=pal[i].g,b1=pal[i].b;
				int r2=pal[j].r,g2=pal[j].g,b2=pal[j].b;
				int r=63-(63-r1)*(63-r2)/63;
				int g=63-(63-g1)*(63-g2)/63;
				int b=63-(63-b1)*(63-b2)/63;
				int n=0,dist=64*64*3;

				for(int c=128;c<255;c++)
				if((pal[c].r-r)*(pal[c].r-r)+(pal[c].g-g)*(pal[c].g-g)+(pal[c].b-b)*(pal[c].b-b)<dist)
				{
					n=c;
					dist=(pal[c].r-r)*(pal[c].r-r)+(pal[c].g-g)*(pal[c].g-g)+(pal[c].b-b)*(pal[c].b-b);
				}

				cmap->data[i][j]=n;

//				int v=i+j-192;
//				if(v>255) v=255;
//				cmap->data[i][j]=v;
			}
			break;
		}
		else
		{
			if(i==0) cmap->data[i][j]=j; // background is always overdrawn
			else if(j==0) cmap->data[i][j]=i;
			else if((i>>5)==1) cmap->data[i][j]=j; // uh...
			else if((j>>5)==1) cmap->data[i][j]=i;
			else if(j==128) cmap->data[i][j]=i; // worm underside is always overdrawn
			else if(i==128) cmap->data[i][j]=j;
			else // brighter colour is drawn
			{
				int v1=pal[i].r+pal[i].g+pal[i].b;
				int v2=pal[j].r+pal[j].g+pal[j].b;
				cmap->data[i][j]=v1>v2?i:j;
			}
		}
	}
}



#ifdef _OS_MACOSX_
#include <Carbon/Carbon.h>
#include <objc/objc.h> 
#include <objc/objc-runtime.h> 
#endif

char *get_config_filename()
{
#ifdef _OS_MACOSX_
	char *pathname;
	static char filename[MAX_STRLEN];

	FSRef ref;
	FSFindFolder(kUserDomain,kPreferencesFolderType,kCreateFolder,&ref);
	CFURLRef url=CFURLCreateFromFSRef(kCFAllocatorDefault,&ref);
	CFStringRef path=CFURLCopyFileSystemPath(url,kCFURLPOSIXPathStyle);
	pathname=(char *)objc_msgSend((id)path,sel_getUid("fileSystemRepresentation"));
	CFRelease(path);
	CFRelease(url);

	strncpy(filename,pathname,MAX_STRLEN);
	strncat(filename,"/swear.cfg",MAX_STRLEN);

	return filename;
#else
	return "swear.cfg";
#endif
}

/*char *get_resource_filename(char *file)
{
#ifdef _OS_MACOSX_
	char *filename;

	if(!CFBundleGetMainBundle()) return file;
	CFStringRef cffile=CFStringCreateWithCString(kCFAllocatorDefault,file,kCFStringEncodingISOLatin1);
	CFURLRef url=CFBundleCopyResourceURL(CFBundleGetMainBundle(),cffile,NULL,NULL);
	CFStringRef path=CFURLCopyFileSystemPath(url,kCFURLPOSIXPathStyle);
	filename=(char *)objc_msgSend((id)path,sel_getUid("fileSystemRepresentation")); 
	CFRelease(path);
	CFRelease(url);
	CFRelease(cffile);

	return filename;
#else
	return file;
#endif
}*/

