#include <stdio.h>
#include <string.h>
#include <time.h>
#include <goa/allegro.h>
#include <goa/menu.h>

#include "main.h"
#include "playfield.h"
#include "game.h"
#include "swearconfig.h"



starfield s;



playfield::playfield()
{
	name="Unnamed"; ident=0;
	onesided=false;

	prepared=false;

	point=0;
	normal=0;
	line=0;
	pp=0;
}

playfield::~playfield()
{
	unprepare();
}

void playfield::prepare()
{
	switch(c->vismode)
	{
		default:
		case VISMODE_WIREFRAME:
			preparefield();
		break;

		case VISMODE_DOTS:
			playfield::preparefield();
		break;
	}

	prepared=true;
}

void playfield::preparefield()
{
	vector dummy;

	npoints=128*c->detailmult[c->detaillevel]*c->detailmult[c->detaillevel];
	nlines=0;

	allocpoints();

	firstprogress();

	for(int i=0;i<npoints;i++)
	{
		random_pos(point[i],normal[i],dummy);

		if((i&127)==0) showprogress(i,npoints);
	}
}

void playfield::unprepare()
{
	delete point; point=0;
	delete normal; normal=0;
	delete line; line=0;
	delete pp; pp=0;

	prepared=false;
}

void playfield::allocpoints()
{
	point=new vector[npoints];
	normal=new vector[npoints];
	line=nlines?new wireframe[nlines]:0;
	pp=new ppoint[npoints];
}

void playfield::recalc(vector &pos,vector &dir,vector &normal,vector &prev)
{
	recalc_pos(pos,normal,prev);

	dir.plane_project(normal);
	dir.normalize(); // project heading onto plane + normalize
}

void playfield::random(vector &pos,vector &dir,vector &normal,vector &prev)
{
	random_pos(pos,normal,prev);

	dir=vector::rand_sphere(); // random heading
	dir.plane_project(normal); // project onto surface
	dir.normalize();
}

void playfield::recalc_pos(vector &pos,vector &normal,vector &prev) {}

void playfield::random_pos(vector &pos,vector &normal,vector &prev) {}

void playfield::draw()
{
	s.live();
	s.draw();

	for(int i=0;i<npoints;i++)
	pp[i].col=g->project(point[i],normal[i],pp[i].x,pp[i].y,onesided);

	switch(c->vismode)
	{
		case VISMODE_WIREFRAME:
			if(c->antialias)
			{
				for(int i=0;i<nlines;i++)
				if(pp[line[i].a].col>=0&&pp[line[i].b].col>=0) antialias_line8(bm,
				pp[line[i].a].x,pp[line[i].a].y,
				pp[line[i].b].x,pp[line[i].b].y,
				64,(pp[line[i].a].col+pp[line[i].b].col)/2);
			}
			else
			{
				for(int i=0;i<nlines;i++)
				if(pp[line[i].a].col>=0&&pp[line[i].b].col>=0) ::line(bm,
				pp[line[i].a].x,pp[line[i].a].y,
				pp[line[i].b].x,pp[line[i].b].y,
				(pp[line[i].a].col+pp[line[i].b].col)/2+64);
			}
		break;

		case VISMODE_DOTS:
			for(int i=0;i<npoints;i++)
			if(pp[i].col>=0) putpixel(bm,pp[i].x,pp[i].y,pp[i].col+64);
		break;
	}

//	for(int i=0;i<npoints;i++)
//	putpixel(bm,pp[i].x,pp[i].y,2);
}



void playfield::firstprogress()
{
	textout_centre(screen,victor_12,"Preparing...",SCREEN_W/2-1,SCREEN_H/2-12,1);

	int x=(SCREEN_W-PROGRESS_WIDTH)/2;
	int y=SCREEN_H/2+1;

	rect(screen,x,y,x+PROGRESS_WIDTH-1,y+PROGRESS_HEIGHT-1,1);
}

void playfield::showprogress(int prog,int end)
{
	int x=(SCREEN_W-PROGRESS_WIDTH)/2;
	int y=SCREEN_H/2+1;
	int size=(PROGRESS_WIDTH-4)*prog/end;

	if(size)
	rectfill(screen,x+2,y+2,x+2+size-1,y+PROGRESS_HEIGHT-3,1);
}



void playfield::loadhighscores()
{
	if(!ident)
	{
		for(int i=0;i<NUM_TYPES;i++)
		for(int j=0;j<NUM_SCORES;j++)
		{
			strcpy(high[i][j].name,"Nobody");
			high[i][j].score=0;
			high[i][j].playtime=0;
			high[i][j].time=0;
		}
		return;
	}

	set_config_file(get_config_filename()); // just to make sure

	for(int i=0;i<NUM_TYPES;i++)
	{
		char sec[MAX_STRLEN];

		if(i) sprintf(sec,"%s%d",ident,i);
		else sprintf(sec,"%s",ident);

		for(int j=0;j<NUM_SCORES;j++)
		{
			char str[MAX_STRLEN];

			sprintf(str,"name%d",j);
			strcpy(high[i][j].name,get_config_string(sec,str,"Nobody"));

			sprintf(str,"score%d",j);
			high[i][j].score=get_config_int(sec,str,0);

			sprintf(str,"playtime%d",j);
			high[i][j].playtime=get_config_int(sec,str,0);

			sprintf(str,"time%d",j);
			high[i][j].time=get_config_int(sec,str,0);
		}
	}
}

void playfield::savehighscores()
{
	if(!ident) return;

	set_config_file(get_config_filename()); // just to make sure

	for(int i=0;i<NUM_TYPES;i++)
	{
		char sec[MAX_STRLEN];

		if(i) sprintf(sec,"%s%d",ident,i);
		else sprintf(sec,"%s",ident);

		for(int j=0;j<NUM_SCORES;j++)
		{
			char str[MAX_STRLEN];

			sprintf(str,"name%d",j);
			set_config_string(sec,str,high[i][j].name);

			sprintf(str,"score%d",j);
			set_config_int(sec,str,high[i][j].score);

			sprintf(str,"playtime%d",j);
			set_config_int(sec,str,high[i][j].playtime);

			sprintf(str,"time%d",j);
			set_config_int(sec,str,high[i][j].time);
		}
	}
}



bool playfield::ishighscore(int type,int score)
{
	if(!ident) return(false);
	return(score>high[type][NUM_SCORES-1].score);
}

int playfield::inserthighscore(int type,char *player,int score,int playtime,int time)
{
	int n=-1;
	for(int i=0;i<NUM_SCORES;i++)
	if(score>high[type][i].score)
	{
		n=i;
		break;
	}

	if(n==-1) return(-1);

	for(int i=NUM_SCORES-1;i!=n;i--)
	high[type][i]=high[type][i-1];

	strcpy(high[type][n].name,player);
	high[type][n].score=score;
	high[type][n].playtime=playtime;
	high[type][n].time=time;

	return(n);
}

void playfield::showhighscores(int type,int highlight)
{
	m_menu *m=new m_menu();

	m->insert(
		new m_label(name,&look_hi),
		new g_spacer(0,4),
		0
	);
	for(int i=0;i<NUM_SCORES;i++)
	{
		char str1[MAX_STRLEN],str2[MAX_STRLEN],str3[MAX_STRLEN];

		sprintf(str1,"%s   ",high[type][i].name);
		sprintf(str2,"%d",high[type][i].score);
		strftime(str3,MAX_STRLEN,"%a %b %e %H:%M:%S %y ",
		localtime((time_t *)&high[type][i].time));
		sprintf(str3+strlen(str3),"(Play time: %d s)",high[type][i].playtime);

		m_dual *d;
		if(i==highlight) d=new m_dual(str1,str2,&look_hi);
		else d=new m_dual(str1,str2,&look_ni);
		d->clear_flag(GUIFLAG_ACTIVE);

		m->insert(d,
			new m_label(str3,&look_small_ni),
		0);
	}
	m->insert(
		new g_spacer(0,4),
		new m_text("Ok",-1,0),
		new m_text("Clear highscores",1,0),
		0
	);

	m->place=GUIPLACE_CENTER|GUIPLACE_MIDDLE;
	m->set_flag(GUIFLAG_NOMOUSE);

	if(m->run()==1)
	{
		m_menu *r=new m_menu(
			new m_label("Really clear highscores?",&look_ni),
			new g_spacer(0,4),
			new m_text("No",-1,&look_ni),
			new m_text("Yes",1,&look_ni),
			0
		);
		r->place=GUIPLACE_CENTER|GUIPLACE_MIDDLE;
		r->set_flag(GUIFLAG_NOMOUSE);

		if(r->run()==1)
		{
			for(int i=0;i<NUM_SCORES;i++)
			{
				strcpy(high[type][i].name,"Nobody");
				high[type][i].score=0;
				high[type][i].playtime=0;
				high[type][i].time=0;
			}
			savehighscores();

			showhighscores(type,-1);
		}
	}

	delete m;
}

void playfield::mergehighscores(char *file)
{
	set_config_file(file);

	highscore h;
	for(int i=0;i<NUM_TYPES;i++)
	{
		char sec[MAX_STRLEN];

		if(i) sprintf(sec,"%s%d",ident,i);
		else sprintf(sec,"%s",ident);

		for(int j=0;j<NUM_SCORES;j++)
		{
			char str[MAX_STRLEN];

			sprintf(str,"name%d",j);
			strcpy(h.name,get_config_string(sec,str,"Nobody"));

			sprintf(str,"score%d",j);
			h.score=get_config_int(sec,str,0);

			sprintf(str,"playtime%d",j);
			h.playtime=get_config_int(sec,str,0);

			sprintf(str,"time%d",j);
			h.time=get_config_int(sec,str,0);

			bool unique=true;

			for(int k=0;k<NUM_SCORES;k++)
			if(
				!strcmp(high[i][k].name,h.name)&&
				high[i][k].score==h.score&&
				high[i][k].playtime==h.playtime&&
				high[i][k].time==h.time
			) { unique=false; break; }

			if(unique)
			if(inserthighscore(i,h.name,h.score,h.playtime,h.time)==-1) break;
		}
	}

	savehighscores();
}



// starfield

starfield::starfield()
{
	for(int i=0;i<STAR_MAXNUM;i++) stars[i]=vector::rand_sphere();
}

starfield::~starfield()
{
}

void starfield::live()
{
	if(!c->starspeed) return;

	int num=(c->stars*STAR_MAXNUM)/STARS_SILLY;
	vector vel=vector(0,0,
		double(c->starspeed*c->starspeed*STAR_MAXSPEED)
		/STARSPEED_SILLY/STARSPEED_SILLY*g->dt
	);

	for(int i=0;i<num;i++)
	{
		stars[i]+=vel;
		if(stars[i].lengthsq()>1)
		{
			do
			{
				stars[i].x=2*drand()-1;
				stars[i].y=2*drand()-1;
			} while(stars[i].x*stars[i].x+stars[i].y*stars[i].y>1);
			stars[i].z=-sqrt(1-stars[i].x*stars[i].x-stars[i].y*stars[i].y);
		}
	}
}

void starfield::draw()
{
	int num=(c->stars*STAR_MAXNUM)/STARS_SILLY;

	for(int i=0;i<num;i++)
	{
		vector rot,r=stars[i];
		int x_s,y_s;
		int col;

		rot.z=-r.dot(g->cam_current.z_axis());
		
		if(rot.z>0)
		{
			rot.y=-r.dot(g->cam_current.y_axis());
			rot.x=-r.dot(g->cam_current.x_axis());
		
			double sc=CAMERA_SCALE/rot.z;

			x_s=int(rot.x*sc+double(SCREEN_W-1)/2);
			y_s=int(rot.y*sc+double(SCREEN_H-1)/2);
		
			col=int((1-r.length())*32)+32;

			putpixel(bm,x_s,y_s,col);
		}
	}
}
